@@ -4,6 +4,8 @@
  *
  * Serial communication layer.
  *
+ * $Id: serial.c,v 1.88 2003/07/10 21:35:38 swestin Exp $
+ *
  ****************************************************************************/
 
 /****************************************************************************
@@ -12,7 +14,7 @@
  *
  ****************************************************************************/
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -59,28 +61,41 @@
 # define MIN(a, b) ((a) < (b) ? (a) : (b))
 #endif
 
+/**
+ * serial_flush_input
+ * @gdev: serial port to use
+ *
+ * Dummy function.
+ *
+ */
 void
 serial_flush_input (GPPort *gdev)
 {
 }
 
+/**
+ * serial_flush_output
+ * @gdev: serial port to use
+ *
+ * Dummy function.
+ *
+ */
 void
 serial_flush_output (GPPort *gdev)
 {
 }
 
-/*****************************************************************************
- *
+/**
  * canon_serial_change_speed
+ * @gdev: serial port to use
+ * @speed: the new speed
  *
- * change the speed of the communication.
+ * Changes the speed of the communication.
  *
- * speed - the new speed
+ * Returns: 1 on success.
+ *          0 on any error.
  *
- * Returns 1 on success.
- * Returns 0 on any error.
- *
- ****************************************************************************/
+ */
 
 int
 canon_serial_change_speed (GPPort *gdev, int speed)
@@ -98,18 +113,18 @@ canon_serial_change_speed (GPPort *gdev, int speed)
 }
 
 
-/*****************************************************************************
- *
+/**
  * canon_serial_get_cts
+ * @gdev: serial port to use
  *
  * Gets the status of the CTS (Clear To Send) line on the serial port.
  *
  * CTS is "1" when the camera is ON, and "0" when it is OFF.
  *
- * Returns 1 on CTS high.
- * Returns 0 on CTS low.
+ * Returns: 1 on CTS high.
+ *          0 on CTS low.
  *
- ****************************************************************************/
+ */
 int
 canon_serial_get_cts (GPPort *gdev)
 {
@@ -119,18 +134,15 @@ canon_serial_get_cts (GPPort *gdev)
 	return (level);
 }
 
-/*****************************************************************************
- *
+/**
  * canon_serial_init
+ * @camera: Camera object to initialize
  *
- * Initializes the given serial or USB device.
+ * Initializes the given serial device by setting speed, parity, etc.
  *
- * devname - the name of the device to open
+ * Returns: %GP_OK
  *
- * Returns 0 on success.
- * Returns -1 on any error.
- *
- ****************************************************************************/
+ */
 
 int
 canon_serial_init (Camera *camera)
@@ -154,20 +166,18 @@ canon_serial_init (Camera *camera)
 	return GP_OK;
 }
 
-/*****************************************************************************
- *
+/**
  * canon_serial_send
+ * @camera: Camera object to work with
+ * @buf:    the raw data buffer to send
+ * @len:    the length of the buffer
+ * @sleep:  time in usec to wait between characters
  *
  * Send the given buffer with given length over the serial line.
  *
- * buf   - the raw data buffer to send
- * len   - the length of the buffer
- * sleep - time in usec to wait between characters
+ * Returns: 0 on success, -1 on error.
  *
- * Returns 0 on success, -1 on error.
- *
- ****************************************************************************/
-
+ */
 int
 canon_serial_send (Camera *camera, const unsigned char *buf, int len, int sleep)
 {
@@ -190,7 +200,12 @@ canon_serial_send (Camera *camera, const unsigned char *buf, int len, int sleep)
 
 
 /**
+ * serial_set_timeout
+ * @gdev: serial port to use
+ * @to:   timeout in milliseconds
+ *
  * Sets the timeout, in miliseconds.
+ *
  */
 void
 serial_set_timeout (GPPort *gdev, int to)
@@ -198,17 +213,17 @@ serial_set_timeout (GPPort *gdev, int to)
 	gp_port_set_timeout (gdev, to);
 }
 
-/*****************************************************************************
- *
+/**
  * canon_serial_get_byte
+ * @gdev: serial port to use
  *
- * Get the next byte from the serial line.
+ * Gets the next byte from the serial line.
  * Actually the function reads chunks of data and keeps them in a cache.
  * Only one byte per call will be returned.
  *
- * Returns the byte on success, -1 on error.
+ * Returns: the byte on success, -1 on error.
  *
- ****************************************************************************/
+ */
 int
 canon_serial_get_byte (GPPort *gdev)
 {
@@ -238,6 +253,17 @@ canon_serial_get_byte (GPPort *gdev)
 
 /* ------------------------- Frame-level processing ------------------------- */
 
+/**
+ * canon_serial_send_frame
+ * @camera: Camera object to work with
+ * @pkt: Data to send to camera
+ * @len: Length of packet
+ *
+ * Sends a frame of data to camera
+ *
+ * Returns: 1 if canon_serial_send() succeeds, 0 if it fails
+ *
+ */
 int
 canon_serial_send_frame (Camera *camera, const unsigned char *pkt, int len)
 {
@@ -266,7 +292,17 @@ canon_serial_send_frame (Camera *camera, const unsigned char *pkt, int len)
 	return !canon_serial_send (camera, buffer, p - buffer, USLEEP2);
 }
 
-
+/**
+ * canon_serial_recv_frame
+ * @camera: Camera object to work with
+ * @len:    to receive the length of the buffer
+ *
+ * Receive a frame from the camera
+ *
+ * Returns: a buffer containing a frame from the camera, or NULL on error.
+ *          On success, @len will contain the length of the buffer.
+ *
+ */
 unsigned char *
 canon_serial_recv_frame (Camera *camera, int *len)
 {
@@ -305,7 +341,21 @@ canon_serial_recv_frame (Camera *camera, int *len)
 
 /* ------------------------ Packet-level processing ------------------------- */
 
-
+/**
+ * canon_serial_send_packet
+ * @camera: Camera object to work with
+ * @type: 
+ * @seq: 
+ * @pkt: data to send to camera
+ * @len: length of data
+ *
+ * frames a packet (generates CRC, packs with sequence number and
+ * length) and sends it to the camera through the serial port using
+ * canon_serial_send_frame().
+ *
+ * Returns: status from canon_serial_send_frame()
+ *
+ */
 int
 canon_serial_send_packet (Camera *camera, unsigned char type, unsigned char seq,
 			  unsigned char *pkt, int len)
@@ -338,6 +388,22 @@ canon_serial_send_packet (Camera *camera, unsigned char type, unsigned char seq,
 	return canon_serial_send_frame (camera, hdr, len + PKT_HDR_LEN + 2);
 }
 
+/**
+ * canon_serial_recv_packet
+ * @camera: Camera object to work with
+ * @type: Type of packet
+ * @seq: Sequence number of packet
+ * @len: length of data received
+ *
+ * Receives a packet from the serial port using
+ * canon_serial_send_frame(), decodes frame information (type,
+ * sequence number, and length), and returns it stripped of frame
+ * information.
+ *
+ * Returns: packet data (or NULL if failure). Type in @type, sequence
+ *   number in @seq, and length in @len.
+ *
+ */
 unsigned char *
 canon_serial_recv_packet (Camera *camera, unsigned char *type, unsigned char *seq, int *len)
 {
@@ -382,9 +448,12 @@ canon_serial_recv_packet (Camera *camera, unsigned char *type, unsigned char *se
 
 
 /**
+ * canon_serial_wait_for_ack
+ * @camera: Camera object to work with
+ *
  * Waits for an "ACK" from the camera.
  *
- * Return values:
+ * Returns:
  *  1 : ACK received
  *  0 : communication error (no reply received for example)
  * -1 : NACK received.
@@ -452,15 +521,18 @@ canon_serial_wait_for_ack (Camera *camera)
 }
 
 /**
+ * canon_serial_send_msg
+ * @camera: Camera object to work with
+ * @mtype:  message type.
+ * @dir:    direction.
+ * @ap:     message payload (list of arguments, see 'man va_start'
+ *
  * Sends a message to the camera.
  *
- * See the "Protocol" file for an explanation of the various
- * elements needed to create a message.
- *
- * Arguments:
- *  mtype : message type.
- *  dir   : direction.
- *  ap    : message payload (list of arguments, see 'man va_start'
+ * Returns:
+ *   -1 on error
+ *    0 if canon_serial_send_packet() fails
+ *    1 on good ACK received
  */
 static int
 canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, va_list * ap)
@@ -563,18 +635,21 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
 }
 
 /**
- * Receive a message from the camera.
+ * canon_serial_recv_msg
+ * @camera: Camera object to work with
+ * @mtype:  message type.
+ * @dir:    direction.
+ * @total:  payload length (set by this function).
+ * @context: context for error reporting
+ *
+ * Receives a message from the camera.
  *
  * See the "Protocol" file for an explanation of the various
  * elements needed to handle a message.
  *
- * Arguments:
- *  mtype : message type.
- *  dir   : direction.
- *  total : payload length (set by this function).
- *
  * Returns:
- *  char* : pointer to the message payload.
+ *  char* pointer to the message payload; NULL on failure.
+ *
  */
 unsigned char *
 canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, int *total,
@@ -726,11 +801,12 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, i
 
 /**
  * canon_serial_dialogue:
- *
+ * @camera: camera with which to communicate
+ * @context: context for error reporting
  * @mtype : type
  * @dir   : direction
  * @len   : length of the received payload
- * @...   : The rest of the arguments will be put together to
+ * @Varargs: The rest of the arguments will be put together to
  *          fill up the payload of the request message.
  *
  * Higher level function: sends a message and waits for a
@@ -743,7 +819,9 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, i
  * Example: To send a string called "name" :
  * canon_serial_dialogue(0x05,0x12,&len,name,strlen(name)+1,NULL);
  *
- **/
+ * Returns: buffer received from canon_serial_recv_msg(), NULL if failure
+ *
+ */
 unsigned char *
 canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
 		       unsigned char dir, int *len, ...)
@@ -801,9 +879,11 @@ canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
 /**
  * canon_serial_end:
  * @camera: the camera to switch off
- * @Returns: GP_OK
  *
- * Switches the #camera off
+ * Switches the @camera off
+ *
+ * Returns: %GP_OK
+ *
  */
 int
 canon_serial_end (Camera *camera)
@@ -816,11 +896,13 @@ canon_serial_end (Camera *camera)
 /**
  * canon_serial_off:
  * @camera: the camera to switch off
- * @Returns: GP_OK
  *
  * Switches the #camera off, and resets the serial driver to 9600 bauds,
  * in order to be ready to switch the camera back on again if wanted.
  * Should better be named psa50_serial_off
+ *
+ * Returns: %GP_OK
+ *
  */
 int
 canon_serial_off (Camera *camera)
@@ -833,9 +915,13 @@ canon_serial_off (Camera *camera)
 
 
 
-/*
- * print a message corresponding
+/**
+ * canon_serial_error_type
+ * @camera: Camera object to work with
+ *
+ * logs a debug message corresponding
  * to the error encountered
+ *
  */
 void
 canon_serial_error_type (Camera *camera)
@@ -853,8 +939,17 @@ canon_serial_error_type (Camera *camera)
 	}
 }
 
-/*
- * Upload to Camera via serial
+/**
+ * canon_serial_put_file
+ * @camera: Camera object to work with
+ * @file: CameraFile object to upload
+ * @destname: name file should have on camera
+ * @destpath: pathname for directory to put file
+ * @context: context for error reporting
+ *
+ * Uploads file to @camera via serial port
+ *
+ * Returns: gphoto2 error code
  *
  */
 int
@@ -922,6 +1017,19 @@ canon_serial_put_file (Camera *camera, CameraFile *file, char *destname, char *d
 	return GP_OK;
 }
 
+/**
+ * canon_serial_get_file:
+ * @camera: camera to lock keys on
+ * @name: name of file to fetch
+ * @length: to receive length of image data
+ * @context: context for error reporting
+ *
+ * Get a file from a USB_connected Canon camera.
+ *
+ * Returns: buffer containing file data (or NULL on failure); length
+ * in @length.
+ *
+ */
 unsigned char *
 canon_serial_get_file (Camera *camera, const char *name, int *length, GPContext *context)
 {
@@ -985,6 +1093,19 @@ canon_serial_get_file (Camera *camera, const char *name, int *length, GPContext
 	return NULL;
 }
 
+/**
+ * canon_serial_get_dirents:
+ * @camera: camera to initialize
+ * @dirent_data: to receive directory data
+ * @dirents_length: to receive length of @dirent_data
+ * @path: pathname of directory to list
+ * @context: context for error reporting
+ *
+ * Lists a directory.
+ *
+ * Returns: gphoto2 error code
+ *
+ */
 int
 canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 			  unsigned int *dirents_length, const char *path, GPContext *context)
@@ -994,21 +1115,21 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 
 	*dirent_data = NULL;
 
-	/* fetch all directory entrys, the first one is a little special */
+	/* fetch all directory entries, the first one is a little special */
 	p = canon_serial_dialogue (camera, context, 0xb, 0x11, dirents_length, "", 1, path,
 				   strlen (path) + 1, "\x00", 2, NULL);
 	if (p == NULL) {
 		gp_context_error (context,
-				  "canon_serial_get_dirents: "
-				  "canon_serial_dialogue failed to fetch directory entrys");
+				  _("canon_serial_get_dirents: "
+				  "canon_serial_dialogue failed to fetch directory entries"));
 		return GP_ERROR;
 	}
 
 	/* In the RS232 implementation, we should never get less than 5 bytes */
 	if (*dirents_length < 5) {
 		gp_context_error (context,
-				  "canon_serial_get_dirents: "
-				  "Initial dirent packet too short (only %i bytes)",
+				  _("canon_serial_get_dirents: "
+				  "Initial dirent packet too short (only %i bytes)"),
 				  *dirents_length);
 		return GP_ERROR;
 	}
@@ -1027,8 +1148,8 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 	data = malloc (mallocd_bytes);
 	if (!data) {
 		gp_context_error (context,
-				  "canon_serial_get_dirents: "
-				  "Could not allocate %i bytes of memory", mallocd_bytes);
+				  _("canon_serial_get_dirents: "
+				  "Could not allocate %i bytes of memory"), mallocd_bytes);
 		return GP_ERROR_NO_MEMORY;
 	}
 
@@ -1040,15 +1161,15 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 
 	/* p[4] indicates this is not the last packet,
 	 * read additional packets until there are no more
-	 * directory entrys to read
+	 * directory entries to read
 	 */
 	while (!p[4]) {
 		GP_DEBUG ("p[4] is %i", (int) p[4]);
 		p = canon_serial_recv_msg (camera, 0xb, 0x21, dirents_length, context);
 		if (p == NULL) {
 			gp_context_error (context,
-					  "canon_serial_get_dirents: "
-					  "Failed to read another directory entry");
+					  _("canon_serial_get_dirents: "
+					  "Failed to read another directory entry"));
 			free (data);
 			data = NULL;
 			return GP_ERROR;
@@ -1065,8 +1186,8 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 		 */
 		if (*dirents_length - 5 < CANON_MINIMUM_DIRENT_SIZE) {
 			gp_context_error (context,
-					  "canon_serial_get_dirents: "
-					  "Truncated directory entry received");
+					  _("canon_serial_get_dirents: "
+					  "Truncated directory entry received"));
 			free (data);
 			data = NULL;
 			return GP_ERROR;
@@ -1087,13 +1208,13 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 			mallocd_bytes += MAX (1024, *dirents_length);
 
 			/* check if we are reading unrealistic ammounts
-			 * of directory entrys so that we don't loop
+			 * of directory entries so that we don't loop
 			 * forever. 1024 * 1024 is picked out of the blue.
 			 */
 			if (mallocd_bytes > 1024 * 1024) {
 				gp_context_error (context,
-						  "canon_serial_get_dirents: "
-						  "Too many dirents, we must be looping.");
+						  _("canon_serial_get_dirents: "
+						  "Too many dirents, we must be looping."));
 				free (data);
 				data = NULL;
 				return GP_ERROR;
@@ -1102,9 +1223,9 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 			temp_ch = realloc (data, mallocd_bytes);
 			if (!temp_ch) {
 				gp_context_error (context,
-						  "canon_serial_get_dirents: "
+						  _("canon_serial_get_dirents: "
 						  "Could not resize dirent buffer "
-						  "to %i bytes", mallocd_bytes);
+						  "to %i bytes"), mallocd_bytes);
 				free (data);
 				data = NULL;
 				return GP_ERROR;
@@ -1127,10 +1248,13 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 /**
  * canon_serial_ready:
  * @camera: camera to get ready
- * @Returns: gphoto2 error code
+ * @context: context for error reporting
  *
  * serial part of canon_int_ready
- **/
+ *
+ * Returns: gphoto2 error code
+ *
+ */
 int
 canon_serial_ready (Camera *camera, GPContext *context)
 {
@@ -1240,19 +1364,26 @@ canon_serial_ready (Camera *camera, GPContext *context)
 
 	camera->pl->first_init = 0;
 
-	/* Compare what the camera identified itself as with our list of known models */
+	/* Compare what the camera identified itself as with our list
+	 * of known models
+	 *
+	 * We iterate over the model list testing id_str, even if we
+	 * don't actually use id_str, but serial_id_string.
+	 */
 	for (i = 0; models[i].id_str != NULL; i++) {
-		if (!strcmp (models[i].id_str, cam_id_str)) {
-			GP_DEBUG ("canon_usb_identify: Serial ID string matches '%s'",
-				  models[i].id_str);
-			gp_context_status (context, "Detected a %s", models[i].id_str);
+		if ((models[i].serial_id_string != NULL) && 
+		    !strcmp (models[i].serial_id_string, cam_id_str)) {
+			GP_DEBUG ("canon_serial_ready: Serial ID string matches '%s'",
+				  models[i].serial_id_string);
+			gp_context_status (context, _("Detected a \"%s\" aka \"%s\""), 
+					   models[i].id_str, models[i].serial_id_string);
 			camera->pl->md = (struct canonCamModelData *) &models[i];
 			break;
 		}
 	}
 
 	if (models[i].id_str == NULL) {
-		gp_context_error (context, "Unknown model '%s'", cam_id_str);
+		gp_context_error (context, _("Unknown model \"%s\""), cam_id_str);
 		return GP_ERROR_MODEL_NOT_FOUND;
 	}
 
@@ -1323,8 +1454,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 					  camera->pl->psa50_eot + PKT_HDR_LEN, 0);
 		if (!canon_serial_wait_for_ack (camera)) {
 			gp_context_status (context,
-					   _
-					   ("Error waiting ACK during initialization retrying"));
+					   _("Error waiting for ACK during initialization retrying"));
 		} else
 			break;
 	}
@@ -1345,16 +1475,18 @@ canon_serial_ready (Camera *camera, GPContext *context)
 
 /**
  * canon_serial_get_thumbnail:
- *
  * @camera: camera to work on
  * @name: file name (complete canon path) of file to get thumbnail for
  * @data: pointer to data pointer
  * @length: pointer to data length
- * @Returns: GP_ERROR code
+ * @context: context for error reporting
  *
  * This is just the serial specific part extracted from the older
  * canon_get_thumbnail() routine. 
- **/
+ *
+ * Returns: gphoto2 error code
+ *
+ */
 int
 canon_serial_get_thumbnail (Camera *camera, const char *name, unsigned char **data,
 			    int *length, GPContext *context)
@@ -1370,7 +1502,7 @@ canon_serial_get_thumbnail (Camera *camera, const char *name, unsigned char **da
 
 	if (camera->pl->receive_error == FATAL_ERROR) {
 		gp_context_error (context,
-				  "ERROR: can't continue a fatal error condition detected");
+				  _("ERROR: a fatal error condition was detected, can't continue "));
 		return GP_ERROR;
 	}
 
@@ -1386,7 +1518,7 @@ canon_serial_get_thumbnail (Camera *camera, const char *name, unsigned char **da
 
 	total = le32atoh (msg + 4);
 	if (total > 2000000) {	/* 2 MB thumbnails ? unlikely ... */
-		gp_context_error (context, "ERROR: %d is too big", total);
+		gp_context_error (context, _("ERROR: %d is too big"), total);
 		return GP_ERROR;
 	}
 	*data = malloc (total);
