@@ -1,6 +1,6 @@
 /* konica.c
  *
- * Copyright (C) 2001 Lutz Mller
+ * Copyright  2001 Lutz Mller
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "konica.h"
 
 #include <stdlib.h>
@@ -53,7 +53,7 @@
 #define CHECK_NULL(r)     {if (!(r)) return (GP_ERROR_BAD_PARAMETERS);}
 
 static int
-k_check (GPContext *c, const char *rb)
+k_check (GPContext *c, const unsigned char *rb)
 {
 	switch ((rb[3] << 8) | rb[2]) {
 	case 0x0000:
@@ -136,9 +136,10 @@ k_check (GPContext *c, const char *rb)
         default:
 		gp_context_error (c, _("The camera has just sent an "
 			"error that has not yet been discovered. Please "
-			"report the following to <gphoto-devel@gphoto.org> "
+			"report the following to %s "
 			"with additional information how you got this error: "
-			"(0x%x,0x%x). Thank you very much!"), rb[2], rb[3]);
+			"(0x%x,0x%x). Thank you very much!"), 
+			MAIL_GPHOTO_DEVEL, rb[2], rb[3]);
 		return (GP_ERROR);
         }
 }
