@@ -1,6 +1,6 @@
 /* library.c (for Largan)
  *
- * Copyright (C) 2002 Hubert Figuiere <hfiguiere@teaser.fr>
+ * Copyright  2002 Hubert Figuiere <hfiguiere@teaser.fr>
  * Code largely borrowed to lmini-0.1 by Steve O Connor
  * With the help of specifications for lmini camera by Largan
  *
@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,8 +44,8 @@ static struct largan_baud_rate {
 	int baud;
 	char value;	/* value to be sent in the protocol */
 } bauds [] = { 
+	{19200, 0x02 },	/* default */
 	{38400, 0x03 },
-	{19200, 0x02 },
 	{ 9600, 0x01 },
 	{ 4800, 0x00 },
 	{    0, 0x00 }
@@ -94,17 +94,21 @@ void largan_pict_alloc_data (largan_pict_info * ptr, uint32_t size)
 
 
 /*
- * Open the camera 
+ * Open the camera
  */
 int largan_open (Camera * camera)
 {
 	int ret;
 	
-	ret = purge_camera (camera);
-	if (ret == GP_ERROR) {
-		return ret;
+        ret = largan_get_num_pict (camera);
+        if (ret < 0){
+                ret = purge_camera (camera);
+                if (ret == GP_ERROR) {
+                        return ret;
+                }
+                ret = wakeup_camera (camera);
 	}
-	return wakeup_camera (camera);
+        return ret;     /* here: number of pictures */
 }
 
 /*
@@ -117,14 +121,16 @@ int largan_get_num_pict (Camera * camera)
 
 	ret = largan_send_command (camera, LARGAN_NUM_PICT_CMD, 0, 0);
 	if (ret < 0) {
+		GP_DEBUG ("largan_send_command() failed: %d\n", ret);
 		return -1;
 	}
-	
 	ret = largan_recv_reply (camera, &reply, &code, NULL);
 	if (ret < 0) {
+		GP_DEBUG ("largan_recv_reply() failed: %d\n", ret);
 		return -1;
 	}
 	if (reply != LARGAN_NUM_PICT_CMD) {
+		GP_DEBUG ("Reply incorrect\n");
 		return -1;
 	}
 	return code;
@@ -135,7 +141,7 @@ int largan_get_num_pict (Camera * camera)
  *
  * return GP_OK if OK. Otherwise < 0
  */
-int largan_get_pict (Camera * camera, largan_pict_type type, 
+int largan_get_pict (Camera * camera, largan_pict_type type,
 		uint8_t index, largan_pict_info * pict)
 {
 	int ret;
@@ -143,7 +149,7 @@ int largan_get_pict (Camera * camera, largan_pict_type type,
 	uint8_t param;
 	uint8_t buf[5];
 	uint32_t pict_size;	/* size of the picture as returned by the camera */
-	
+
 	switch (type) {
 		case LARGAN_PICT:
 			param = 0x01;
@@ -152,21 +158,46 @@ int largan_get_pict (Camera * camera, largan_pict_type type,
 			param = 0x00;
 			break;
 		default:
-			GP_DEBUG ("largan_get_pict(): wrong picture type !\n");
+			GP_DEBUG ("largan_get_pict(): wrong picture type requested !\n");
 			return GP_ERROR;
 	}
 	ret = largan_send_command (camera, LARGAN_GET_PICT_CMD, param, index);
 	if (ret < 0) {
 		return ret;
 	}
-	ret = largan_recv_reply (camera, &reply, &code, NULL);
+/* here we have to receive 7 bytes back 
+ the 2nd contains 00 for thumbnail or 01 for original picture
+ the 3rd contains the picturenumber for an original picture
+ or 00 for an thumbnail for an lowres picture
+ or 01 for an thumbnail for an hires picture
+ bytes 4 5 6 7 contain the number of byte coming in the datastream */
+/* the 1st and the 2nd byte are read here */
 	if (ret < 0) {
-		return ret;
-	}
-	if ((reply != LARGAN_GET_PICT_CMD) || (code != 0x01)) {
+                /* clean up and give it a 2nd chance */
+                wakeup_camera (camera);
+                largan_send_command (camera, LARGAN_GET_PICT_CMD, param, index);
+                GP_DEBUG ("largan_get_pict(): command sent 2nd time\n");
+                ret = largan_recv_reply (camera, &reply, &code, NULL);
+                if (ret < 0) {
+                        /* clean up and give it even a 3rd chance */
+                        wakeup_camera (camera);
+                        sleep (5);
+                        largan_send_command (camera, LARGAN_GET_PICT_CMD, param, index);
+                        GP_DEBUG ("largan_get_pict(): command sent 3rd time\n");
+                        ret = largan_recv_reply (camera, &reply, &code, NULL);
+                        if (ret < 0) {
+                                GP_DEBUG ("largan_get_pict(): timeout after command sent 3rd time\n");
+                                return ret;     /* timeout after 3rd trial */
+                        }
+                }
+        }
+        if ((reply != LARGAN_GET_PICT_CMD) || ((code != 0x01) && (code != 0x00))) {
+                GP_DEBUG ("largan_get_pict(): code != 0x01 && 0x00\n");
 		return GP_ERROR;
 	}
 	
+	/* the remaining 5 bytes are read here */
+
 	ret = gp_port_read (camera->port, buf, sizeof(buf));
 	if (ret < GP_OK) {
 		return ret;
@@ -189,7 +220,8 @@ int largan_get_pict (Camera * camera, largan_pict_type type,
 	}
 	pict->type = type;
 	pict_size = be32atoh (&buf[1]);
-	if (type == LARGAN_PICT) {
+	switch (type) {
+	case LARGAN_PICT: 
 		largan_pict_alloc_data (pict, pict_size);
 		ret = gp_port_read (camera->port, pict->data, pict->data_size);
 		if (ret < GP_OK) {
@@ -200,19 +232,26 @@ int largan_get_pict (Camera * camera, largan_pict_type type,
 			return GP_ERROR;
 		}
 		pict->quality = 0xff;	/* this is not a thumbnail */
-	}
-	else {
-		char * buffer = malloc (pict_size);
-		ret = gp_port_read (camera->port, buffer, pict_size);
-		if (ret < GP_OK) {
-			return ret;
+		break;
+	case LARGAN_THUMBNAIL: 
+		{
+			char * buffer = (char*)malloc(pict_size);
+			ret = gp_port_read (camera->port, buffer, pict_size);
+			if (ret < GP_OK) {
+				return ret;
+			}
+			largan_pict_alloc_data (pict, 19200 + sizeof(BMPheader));
+			memcpy (pict->data, BMPheader, sizeof(BMPheader));
+			/*this is the segfaulty function */
+			largan_ccd2dib (buffer, pict->data + sizeof(BMPheader), 240, 1);
+			
+			free (buffer);
+			pict->quality = buf[0];
+			break;
 		}
-		largan_pict_alloc_data (pict, 19200 + sizeof(BMPheader));
-		memcpy (pict->data, BMPheader, sizeof(BMPheader));
-		largan_ccd2dib (buffer, pict->data + sizeof(BMPheader), 240, 1);
-		
-		free (buffer);
-		pict->quality = buf[0];
+	default:
+		GP_DEBUG ("largan_get_pict(): type not LARGAN_PICT nor LARGAN_THUMBNAIL\n");
+                return GP_ERROR;
 	}
 	return GP_OK;
 }
@@ -225,23 +264,32 @@ int largan_get_pict (Camera * camera, largan_pict_type type,
  *
  * return GP_OK if OK.
  */
-int largan_erase (Camera *camera, int all)
+int largan_erase (Camera *camera, int pict_num)
 {
 	int ret;
+	int num_pix;
 	uint8_t erase_code;
 	uint8_t reply,code;
-	
-	if (all) {
+
+	if (pict_num == 0xff) {
 		erase_code = 0x11;
+		GP_DEBUG ("largan_erase() all sheets \n");
 	}
 	else {
-		erase_code = 0x10;
+		num_pix = largan_get_num_pict (camera);
+		if (pict_num == num_pix) {
+			erase_code = 0x10;
+			GP_DEBUG ("largan_erase() last sheet \n");
+		}
+		else {
+			GP_DEBUG ("Only the last sheet can be erased!\n");
+			return GP_ERROR;
+		}
 	}
 	ret = largan_send_command (camera, LARGAN_BAUD_ERASE_CMD, erase_code, 0);
 	if (ret < 0) {
 		return ret;
 	}
-	
 	ret = largan_recv_reply (camera, &reply, &code, NULL);
 	if (ret < 0) {
 		return ret;
@@ -267,9 +315,9 @@ int largan_capture (Camera *camera)
 	if (ret < 0) {
 		return ret;
 	}
-
 	ret = largan_recv_reply (camera, &reply, &code, &code2);
 	if (ret < 0) {
+		GP_DEBUG ("return ret\n");
 		return ret;
 	}
 	if (reply != LARGAN_CAPTURE_CMD) {
@@ -277,6 +325,7 @@ int largan_capture (Camera *camera)
 		return GP_ERROR;
 	}
 	if (code != code2) {
+		GP_DEBUG ("code != code2\n");
 		return GP_ERROR;
 	}
 	if (code == 0xee) {
@@ -312,7 +361,7 @@ int largan_set_serial_speed (Camera * camera, int speed)
 				return ret;
 			}
 			ret = largan_recv_reply (camera, &reply, &code, NULL);
-			if (ret < 0) {	
+			if (ret < 0) {
 				return ret;
 			}
 			if (reply !=  LARGAN_BAUD_ERASE_CMD) {
@@ -321,7 +370,7 @@ int largan_set_serial_speed (Camera * camera, int speed)
 			if (code != bauds[i].baud) {
 				return ret;
 			}
-			return set_serial_speed (camera, bauds[i].baud);	
+			return set_serial_speed (camera, bauds[i].baud);
 		}
 	}
 	GP_DEBUG ("largan_set_serial_speed(): baud rate not found\n");
@@ -331,45 +380,79 @@ int largan_set_serial_speed (Camera * camera, int speed)
 
 /*
  * received a reply from the camera.
- * can accept NULL on imput.
+ * can accept NULL on input.
  * TODO: check how much we need to read.
  */
-static int largan_recv_reply (Camera * camera, uint8_t *reply, 
+static int largan_recv_reply (Camera * camera, uint8_t *reply,
 		uint8_t *code, uint8_t *code2)
 {
 	int ret;
 	uint8_t packet [4];
+	uint8_t	packet_size = 0;
 	memset (packet, 0, sizeof (packet));
-	
-	ret = gp_port_read (camera->port, packet, sizeof(packet));
+	ret = gp_port_read (camera->port, packet, 1);
 	if (ret < GP_OK) {
 		return ret;
 	}
+	switch (packet[0])
+	{
+	case LARGAN_NUM_PICT_CMD:
+		packet_size = 2;
+		break;
+	case LARGAN_GET_PICT_CMD:
+		packet_size = 2;  
+		break;
+	case LARGAN_BAUD_ERASE_CMD:
+		packet_size = 2;
+		break;
+	case LARGAN_CAPTURE_CMD:
+		packet_size = 3;
+		break;
+	default:
+		packet_size = 0;
+		GP_DEBUG("largan_receive_reply: Unkown reply.\n");
+		break;
+	}
 	if (reply) {
 		*reply = packet[0];
 	}
-	if (code) {
-		*code = packet[1];
+	if (packet_size >= 2) {
+		ret = gp_port_read (camera->port, &packet[1], 1);
+		if (ret < GP_OK) {
+			return ret;
+		}
+		if (code) {
+			*code = packet[1];
+		}
 	}
-	if (code2) {
-		*code2 = packet[2];
+	if (packet_size >= 3) {
+		ret = gp_port_read (camera->port, &packet[2], 1);
+		if (ret < GP_OK) {
+			return ret;
+		}
+		if (code2) {
+			*code2 = packet[2];
+		}
 	}
+
 	return ret;
 }
 
 /*
  * Send a command packet
  */
-static int largan_send_command (Camera * camera, uint8_t cmd, uint8_t param1, 
+static int largan_send_command (Camera * camera, uint8_t cmd, uint8_t param1,
 		uint8_t param2)
 {
+	uint8_t	packet_size = 0;
 	uint8_t packet [3];
 	memset (&packet, 0, sizeof (packet));
-	
+
 	packet [0] = cmd;
 
 	switch (cmd) {
 	case LARGAN_NUM_PICT_CMD:
+		packet_size = 1;
 		break;
 	case LARGAN_GET_PICT_CMD:
 		if ((param1 != 0) && (param1 != 1)) {
@@ -378,6 +461,7 @@ static int largan_send_command (Camera * camera, uint8_t cmd, uint8_t param1,
 		}
 		packet[1] = param1;
 		packet[2] = param2;
+		packet_size = 3;
 		break;
 	case LARGAN_BAUD_ERASE_CMD:
 		if ((param1 > 0x11) || ((param1 > 0x03) && (param1 < 0x10))) {
@@ -385,15 +469,17 @@ static int largan_send_command (Camera * camera, uint8_t cmd, uint8_t param1,
 			return GP_ERROR; /* wrong parameter */
 		}
 		packet[1] = param1;
+		packet_size = 2;
 		break;
 	case LARGAN_CAPTURE_CMD:
+		packet_size = 1;
 		break;
 	default:
 		GP_DEBUG ("unknown command\n");
 		return GP_ERROR; /* unknown command */
 	}
 
-	return gp_port_write (camera->port, packet, sizeof (packet));
+	return gp_port_write (camera->port, packet, packet_size);
 }
 
 
@@ -404,6 +490,7 @@ static int set_serial_speed (Camera * camera, int speed)
 {
 	int ret;
 	GPPortSettings settings;
+	GP_DEBUG ("set_serial_speed() called ***************\n");
 	
 	if (camera->port->type != GP_PORT_SERIAL) {
 		GP_DEBUG ("set_serial_speed() called on non serial port\n");
@@ -411,8 +498,9 @@ static int set_serial_speed (Camera * camera, int speed)
 	}
 	
 	ret = gp_port_get_settings (camera->port, &settings);
-	if (ret < 0)
-		return (ret);
+	if (ret < 0) {
+		return ret;
+	}
 	settings.serial.speed    = speed;
 	
 	ret = gp_port_set_settings (camera->port, settings);
@@ -462,23 +550,19 @@ static int purge_camera (Camera * camera)
 static int wakeup_camera (Camera * camera)
 {
 	int num_pix;
-	int i;
+	/*	int i;*/
 	int ret;
-
-	if (camera->port->type == GP_PORT_SERIAL) {
 	
-		for (i = 0; bauds[i].baud; i++)
-		{
-			ret = set_serial_speed (camera, bauds[i].baud);
-			num_pix = largan_get_num_pict (camera);
-			if (num_pix >= 0)
-			{
-				return GP_OK;
-			}
-			sleep (1);
+	if (camera->port->type == GP_PORT_SERIAL) {
+		ret = set_serial_speed (camera, 4800);	/*wakes camera best*/
+		num_pix = largan_get_num_pict (camera);	/*this wakes*/
+		ret = set_serial_speed (camera, 19200);	/*back to normal*/
+		sleep (1);
+		num_pix = largan_get_num_pict (camera);	/*this wakes*/
+		if (num_pix >= 0){
+			return GP_OK;
 		}
 	}
 	purge_camera (camera);
 	return GP_ERROR;
 }
-
