@@ -1,7 +1,7 @@
 /*
-	Copyright (c) 2000 Mariusz Zynel <mariusz@mizar.org> (gPhoto port)
-	Copyright (C) 2000 Fredrik Roubert <roubert@df.lth.se> (idea)
-	Copyright (C) 1999 Galen Brooks <galen@nine.com> (DC1580 code)
+	Copyright  2000 Mariusz Zynel <mariusz@mizar.org> (gPhoto port)
+	Copyright  2000 Fredrik Roubert <roubert@df.lth.se> (idea)
+	Copyright  1999 Galen Brooks <galen@nine.com> (DC1580 code)
 
 	This file is part of the gPhoto project and may only be used,  modified,
 	and distributed under the terms of the gPhoto project license,  COPYING.
@@ -18,7 +18,7 @@
 	
 */	
  
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <sys/time.h>
@@ -346,7 +346,7 @@ int camera_abilities (CameraAbilitiesList *list) {
         int             result;
 
 	memset (&a, 0, sizeof(a));
-	strcpy(a.model, "Panasonic DC1000");
+	strcpy(a.model, "Panasonic:DC1000");
 	a.status = GP_DRIVER_STATUS_PRODUCTION;
 	a.port		= GP_PORT_SERIAL;
 	a.speed[0] 	= 9600;
