@@ -26,7 +26,7 @@
 /* Boston, MA 02111-1307, USA.                                  */
 /****************************************************************/
 
-#include <config.h>
+#include "config.h"
 
 #include "pccam600.h"
 
@@ -67,7 +67,7 @@ struct models{
   unsigned short idProduct;
   char serial;
 } models[] = {
-  {"Creative PC-CAM600",0x041e,0x400b,0},
+  {"Creative:PC-CAM600",0x041e,0x400b,0},
   {NULL,0,0,0}
 };
 
