@@ -1,24 +1,24 @@
 /* pdc640.c
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
- * Copyright (C) 2002 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright  2001 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright  2002 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,9 +39,30 @@
 
 #define CHECK_RESULT(result) {int r = (result); if (r < 0) return (r);}
 
+#ifdef ENABLE_NLS
+#  include <libintl.h>
+#  undef _
+#  define _(String) dgettext (GETTEXT_PACKAGE, String)
+#  ifdef gettext_noop
+#    define N_(String) gettext_noop (String)
+#  else
+#    define N_(String) (String)
+#  endif
+#else
+#  define textdomain(String) (String)
+#  define gettext(String) (String)
+#  define dgettext(Domain,Message) (Message)
+#  define dcgettext(Domain,Message,Type) (Message)
+#  define bindtextdomain(Domain,Directory) (Domain)
+#  define _(String) (String)
+#  define N_(String) (String)
+#endif
+
+
 typedef enum{
 	pdc640,
-	jd350e
+	jd350e,
+	dlink350f
 } Model;
 
 typedef int postproc_func(int,int,unsigned char*);
@@ -53,6 +74,8 @@ struct _CameraPrivateLibrary{
 	char*			filespec;
 };
 
+static int flip_vertical (int width, int height, unsigned char *rgb);
+static int flip_both     (int width, int height, unsigned char *rgb);
 
 static struct {
 	const char*				model;
@@ -62,11 +85,18 @@ static struct {
 	{"Polaroid Fun Flash 640", 0, 0, {
 		 	pdc640,
 			BAYER_TILE_RGGB,
-			NULL, // add postprocessor here!
+			NULL, /* add postprocessor here! */
 			"pdc640%04i.ppm"
 		}
 	},
-	{"Jenoptik JD350 entrance", 0, 0, {
+	{"Novatech Digital Camera CC30", 0, 0, {
+		 	pdc640,
+			BAYER_TILE_RGGB,
+			NULL, /* add postprocessor here! */
+			"pdc640%04i.ppm"
+		}
+	},
+	{"Jenoptik JD350 entrance", 0x5da, 0x1006, {
 		 	jd350e,
 			BAYER_TILE_BGGR,
 			&jd350e_postprocessing,
@@ -80,6 +110,92 @@ static struct {
 			"jd350v%04i.ppm"
 		}
 	},
+	{"ScanHex SX-35a", 0x797, 0x8901, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&jd350e_postprocessing,
+			"jd350v%04i.ppm"
+		}
+	},
+	{"ScanHex SX-35b", 0x797, 0x8909, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&jd350e_postprocessing,
+			"jd350v%04i.ppm"
+		}
+	},
+	{"ScanHex SX-35c", 0x797, 0x8911, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&jd350e_postprocessing,
+			"jd350v%04i.ppm"
+		}
+	},
+	{"ScanHex SX-35d", 0x84d, 0x1001, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&jd350e_postprocessing,
+			"jd350v%04i.ppm"
+		}
+	},
+	{"Typhoon StyloCam", 0x797, 0x801a, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&flip_vertical,
+			"stylo%04i.ppm"
+		}
+	},
+	{"Trust PowerC@m 350FS", 0x6d6, 0x002e, {
+		 	jd350e,
+			BAYER_TILE_RGGB,
+			&trust350fs_postprocessing,
+			"trust%04i.ppm"
+		}
+	},
+	{"Trust PowerC@m 350FT", 0x6d6, 0x002d, {
+			jd350e,
+			BAYER_TILE_RGGB,
+			&trust350fs_postprocessing,
+			"trust%04i.ppm"
+		}
+	},
+	/* http://www.meade.com/sportsoptics/catalog/captureview/index.html */
+	{"GrandTek ScopeCam", 0x797, 0x801c, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&jd350e_postprocessing_and_flip,
+			"scope%04i.ppm"
+		}
+	},
+	/* http://www.sipixdigital.com/cameras/stylecam/ */
+	/* Ids glanced from downloaded driver / .inf file. */
+	{"SiPix Stylecam", 0xd64, 0x1001, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&flip_both,
+			"style%04i.ppm"
+		}
+	},
+	/* http://www.umax.de/digicam/AstraPix320S.htm */
+	/* reportedly has same ids as SiPix StyleCam and also
+	 * looks identical. */
+	{"UMAX AstraPix 320s", 0xd64, 0x1001, {
+		 	jd350e,
+			BAYER_TILE_BGGR,
+			&flip_both,
+			"astra%04i.ppm"
+		}
+	},
+
+	/* http://www.dlink.com/products/usb/dsc350/ */
+	/* ids from driver download */
+	{"D-Link DSC 350F", 0xd64, 0x1021, {
+		 	dlink350f,
+			BAYER_TILE_BGGR,
+			&flip_vertical,
+			"dlink%04i.ppm"
+		}
+	},
 	{NULL,}
 };
 
@@ -113,10 +229,10 @@ static int
 pdc640_transmit (GPPort *port, char *cmd, int cmd_size,
 				   char *buf, int buf_size)
 {
-	int r;
+	int r, tries;
 
 	if (port->type == GP_PORT_USB) {
-		char xbuf[64];
+		unsigned char xbuf[64];
 		unsigned char xcmd[4];
 		int checksum;
 
@@ -124,18 +240,27 @@ pdc640_transmit (GPPort *port, char *cmd, int cmd_size,
 		memset(xcmd,0,4);memcpy(xcmd,cmd,cmd_size);
 		checksum = (xcmd[0]^0x34)+(xcmd[1]^0xcb)+0x14f+(xcmd[2]^0x67);
 		xcmd[3] = checksum & 0xff;
-		r = gp_port_usb_msg_read( port, 0x10, xcmd[0]|(xcmd[1]<<8),xcmd[2]|(xcmd[3]<<8),xbuf,sizeof(xbuf));
 
+		/* Wait until we get back the echo of the command. */
+		r = gp_port_usb_msg_read( port, 0x10, xcmd[0]|(xcmd[1]<<8),xcmd[2]|(xcmd[3]<<8),xbuf,sizeof(xbuf));
 			/* Sometimes we want to read here, sometimes not.
 		if (r < GP_OK)
 			return r;
 			 */
-		if (buf && buf_size)
-			r = gp_port_read( port, buf, (buf_size + 63) & ~63 );
+		if (buf && buf_size) {
+			/* bulk read things. The JD350v can read in 1 block,
+			 * the Stylocam returns only 64byte blocks here.
+			 */
+			int curr = 0, readsize = (buf_size + 63) & ~63;
+			while (curr < readsize) {
+				r = gp_port_read( port, buf + curr, readsize - curr);
+				if (r < GP_OK) break;
+				curr += r;
+			}
+		}
 		return r;
 	} else {
 		char c;
-		int tries;
 
 		/* In event of a checksum or timing failure, retry */
 		for (tries = 0; tries < PDC640_MAXTRIES; tries++) {
@@ -248,6 +373,7 @@ pdc640_ping_low (GPPort *port)
 	return (GP_OK);
 }
 
+#if 0
 static int
 pdc640_push_button (GPPort *port)
 {
@@ -257,6 +383,7 @@ pdc640_push_button (GPPort *port)
 
 	return (GP_OK);
 }
+#endif
 
 static int
 pdc640_ping_high (GPPort *port)
@@ -357,11 +484,16 @@ pdc640_picinfo (GPPort *port, char n,
 	/* Compression Type */
 	*compression_type = buf[9];
 
-	/* Thumbnail size, width and height */
 	*size_thumb   = buf[25] | (buf[26] << 8) | (buf[27] << 16);
+
 	*width_thumb  = buf[28] | (buf[29] << 8);
 	*height_thumb  = buf[30] | (buf[31] << 8);
 
+	/* Even though it should the be the correct size, the Typhoon
+	 * Stylocam returns junk in there. So calculate for ourselves.
+	 */
+	if (*size_thumb > *width_thumb  * *height_thumb)
+		*size_thumb = *width_thumb  * *height_thumb;
 	return (GP_OK);
 }
 
@@ -465,7 +597,7 @@ pdc640_deltadecode (int width, int height, char **rawdata, int *rawsize)
 			while (pdc640_getbit(*rawdata, &rawofs, *rawsize, &bit) == 1)
 				ones++;
 
-			/* 
+			/*
 			 * Get the delta value
 			 * (size dictated by number of ones)
 			 */
@@ -483,7 +615,7 @@ pdc640_deltadecode (int width, int height, char **rawdata, int *rawsize)
 				val = (col1 += val);
 
 			data[ofs + x] = val << 1;
-		}		
+		}
 	}
 
 	/* Set new buffer */
@@ -513,28 +645,32 @@ pdc640_getpic (Camera *camera, int n, int thumbnail, int justraw,
 
 	/* Evaluate parameters */
 	if (thumbnail) {
-		GP_DEBUG ("Size %d, width %d, height %d",
-			size_thumb, width_thumb, height_thumb);
+		GP_DEBUG ("Size %d, width %d, height %d, comptype %d",
+			size_thumb, width_thumb, height_thumb,
+			(compression_type>>2) & 3);
 
 		*size = size_thumb;
 		width = width_thumb;
 		height = height_thumb;
-		cmd = 0x03;
+		if ((compression_type >> 2) & 3)
+			cmd = 0x02;
+		else
+			cmd = 0x03;
 	} else {
-		GP_DEBUG ("Size %d, width %d, height %d",
-			size_pic, width_pic, height_pic);
+		GP_DEBUG ("Size %d, width %d, height %d, comptype %d",
+			size_pic, width_pic, height_pic, compression_type & 3);
 
 		*size = size_pic;
 		width = width_pic;
 		height = height_pic;
-		if( compression_type == 2 ){
-			cmd = 0x10; // delta compressed
-		}
-		else if( compression_type == 0 ){
-			cmd = 0x00; // uncompressed
-		}
-		else{
-			// unknown compression type
+		switch (compression_type & 3) {
+		case 1:
+		case 2:	cmd = 0x10;	/* delta compressed */
+			break;
+		case 0:	cmd = 0x00;	/* uncompressed */
+			break;
+		default:		/* unknown compression type */
+			GP_DEBUG ("Unknown compression type %d", compression_type & 3);
 			return (GP_ERROR_CORRUPTED_DATA);
 		}
 	}
@@ -543,23 +679,23 @@ pdc640_getpic (Camera *camera, int n, int thumbnail, int justraw,
 	if ((*size <= 0) || (width <= 0) || (height <= 0))
 		return (GP_ERROR_CORRUPTED_DATA);
 
-	/* Allocate the memory */
-	*data = malloc (*size * sizeof (char));
+	/* Allocate the memory, including 64byte align */
+	*data = malloc ((*size + 64) * sizeof (char));
 	if (!*data)
 		return (GP_ERROR_NO_MEMORY);
 
 	/* Get the raw picture */
 	CHECK_RESULT (pdc640_setpic (camera->port, n));
-	CHECK_RESULT (pdc640_transmit_pic (camera->port, cmd, width, thumbnail, 
+	CHECK_RESULT (pdc640_transmit_pic (camera->port, cmd, width, thumbnail,
 					*data, *size));
 
-	if (thumbnail || compression_type == 0 ) {
+	if (thumbnail || (compression_type == 0 )) {
 		/* Process uncompressed data */
 		CHECK_RESULT (pdc640_processtn (width, height,
 						data, *size));
-	} else if( compression_type == 2 ){
+	} else if (compression_type & 3) {
 		/* Image data is delta encoded so decode it */
-		CHECK_RESULT (pdc640_deltadecode (width, height, 
+		CHECK_RESULT (pdc640_deltadecode (width, height,
 						  data, size));
 	}
 
@@ -647,9 +783,10 @@ camera_abilities (CameraAbilitiesList *list)
 
 		if (models[i].vendor) {
 			a.status = GP_DRIVER_STATUS_TESTING;
-			a.port = GP_PORT_USB;
+			a.port = GP_PORT_USB | GP_PORT_SERIAL;
 			a.usb_vendor = models[i].vendor;
 			a.usb_product = models[i].product;
+			a.speed[0] = 0;
 		} else {
 			a.status = GP_DRIVER_STATUS_EXPERIMENTAL;
 			a.port = GP_PORT_SERIAL;
@@ -755,13 +892,14 @@ delete_file_func (CameraFilesystem *fs, const char *folder, const char *file,
 static int
 camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
-	strcpy (about->text, "Download program for Polaroid Fun Flash 640. "
+	strcpy (about->text, _("Download program for GrandTek 98x based cameras. "
 		"Originally written by Chris Byrne <adapt@ihug.co.nz>, "
-		"and adapted for gphoto2 by Lutz Mller "
-		"<urc8@rz.uni-karlsruhe.de>."
+		"and adapted for gphoto2 by Lutz Mueller "
+		"<lutz@users.sf.net>."
 		"Protocol enhancements and postprocessing "
 		" for Jenoptik JD350e by Michael Trawny "
-		"<trawny99@users.sourceforge.net>.");
+		"<trawny99@users.sourceforge.net>."
+		"Bugfixes by Marcus Meissner <marcus@jet.franken.de>."));
 
 	return (GP_OK);
 }
@@ -834,7 +972,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 			    GP_FILE_INFO_HEIGHT | GP_FILE_INFO_TYPE;
 	info->file.width  = width_pic;
 	info->file.height = height_pic;
-/*	
+/*
 	info->file.size   = size_pic;
 */
 	info->file.size   = width_pic * height_pic * 3;
@@ -869,7 +1007,7 @@ camera_init (Camera *camera, GPContext *context)
 	CameraAbilities abilities;
 
 	/*
-	 * First of all, tell gphoto2 about the functions we 
+	 * First of all, tell gphoto2 about the functions we
 	 * implement (especially camera_exit so that everything
 	 * gets correctly cleaned up even in case of error).
 	 */
@@ -932,3 +1070,34 @@ camera_init (Camera *camera, GPContext *context)
 	}
 	return (GP_OK);
 }
+
+static int
+flip_both (int width, int height, unsigned char *rgb)
+{
+	unsigned char *end, c;
+
+	end = rgb + ((width * height) * 3);
+	while (rgb < end) {
+		c = *rgb;
+		*rgb++ = *--end;
+		*end = c;
+	}
+
+	return GP_OK;
+}
+
+static int
+flip_vertical (int width, int height, unsigned char* rgb) {
+	int		i;
+	unsigned char	*buf;
+
+	buf = malloc(width*3);
+	if (!buf) return GP_ERROR_NO_MEMORY;
+	for (i=0;i<height/2;i++) {
+		memcpy(buf,rgb+i*width*3,width*3);
+		memcpy(rgb+i*width*3,rgb+(height-i-1)*width*3,width*3);
+		memcpy(rgb+(height-i-1)*width*3,buf,width*3);
+	}
+	free(buf);
+	return GP_OK;
+}
