@@ -1,6 +1,6 @@
 /* ricoh.c
  *
- * Copyright (C) 2002 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "ricoh.h"
 
 #include <stdlib.h>
@@ -64,8 +64,8 @@
 {							\
 	if (cmd != target) {				\
 		gp_context_error (context, _("Expected %i, got %i. Please "\
-			"report this error to <gphoto-devel@gphoto.org>."),\
-			cmd, target);			\
+			"report this error to %s."),\
+			cmd, target, MAIL_GPHOTO_DEVEL);			\
 		return (GP_ERROR_CORRUPTED_DATA);	\
 	}						\
 }
@@ -75,8 +75,8 @@
 		gp_context_error (context, _("Expected "\
 			"%i bytes, got %i. "		\
 			"Please report this error to "	\
-			"<gphoto-devel@gphoto.org>."),	\
-			target, len);			\
+			"%s."),	\
+			target, len, MAIL_GPHOTO_DEVEL);			\
 		return (GP_ERROR_CORRUPTED_DATA);	\
 	}						\
 }
@@ -171,8 +171,8 @@ ricoh_recv (Camera *camera, GPContext *context, unsigned char *cmd,
 			if (buf[0] != DLE) {
 				gp_context_error (context, _("We expected "
 					"0x%x but received 0x%x. Please "
-					"contact <gphoto-devel@gphoto.org>."),
-						DLE, buf[0]);
+					"contact %s."),
+						DLE, buf[0], MAIL_GPHOTO_DEVEL);
 				return (GP_ERROR_CORRUPTED_DATA);
 			}
 			if (buf[1] != ACK)
@@ -185,8 +185,8 @@ ricoh_recv (Camera *camera, GPContext *context, unsigned char *cmd,
 		default:
 			gp_context_error (context, _("We expected "
 				"0x%x but received 0x%x. Please "
-				"contact <gphoto-devel@gphoto.org>."),
-					STX, buf[1]);
+				"contact %s."),
+					STX, buf[1], MAIL_GPHOTO_DEVEL);
 			return GP_ERROR_CORRUPTED_DATA;
 		}
 
@@ -214,9 +214,8 @@ ricoh_recv (Camera *camera, GPContext *context, unsigned char *cmd,
 						gp_context_error (context,
 							_("Bad characters "
 							"(0x%x, 0x%x). Please "
-							"contact <gphoto-devel"
-							"@gphoto.org>."),
-							data[r], data[r + 1]);
+							"contact %s."),
+							data[r], data[r + 1], MAIL_GPHOTO_DEVEL);
 						return (GP_ERROR_CORRUPTED_DATA);
 					}
 					memmove (&data[r], &data[r +1],
@@ -256,7 +255,7 @@ ricoh_recv (Camera *camera, GPContext *context, unsigned char *cmd,
 			if (ii >= 4) {
 				gp_context_error (context, _("Camera busy. "
 					"If the problem persists, please "
-					"contact <gphoto-devel@gphoto.org>."));
+					"contact %s."), MAIL_GPHOTO_DEVEL);
 				return (GP_ERROR);
 			}
 			continue;
@@ -291,7 +290,7 @@ ricoh_transmit (Camera *camera, GPContext *context, unsigned char cmd,
 			if (++r > 2) {
 				gp_context_error (context, _("Timeout "
 					"even after 2 retries. Please "
-					"contact <gphoto-devel@gphoto.org>."));
+					"contact %s."), MAIL_GPHOTO_DEVEL);
 				return (GP_ERROR_TIMEOUT);
 			}
 			GP_DEBUG ("Timeout! Retrying...");
@@ -307,7 +306,7 @@ ricoh_transmit (Camera *camera, GPContext *context, unsigned char cmd,
 			if (++r > 2) {
 				gp_context_error (context, _("Communication "
 					"error even after 2 retries. Please "
-					"contact <gphoto-devel@gphoto.org>."));
+					"contact %s."), MAIL_GPHOTO_DEVEL);
 				return (GP_ERROR);
 			}
 			continue;
@@ -325,7 +324,7 @@ ricoh_transmit (Camera *camera, GPContext *context, unsigned char cmd,
 			if (++r >= 4) {
 				gp_context_error (context, _("Camera busy. "
 					"If the problem persists, please "
-					"contact <gphoto-devel@gphoto.org>."));
+					"contact %s."), MAIL_GPHOTO_DEVEL);
 				return (GP_ERROR);
 			}
 			continue;
@@ -339,7 +338,7 @@ ricoh_transmit (Camera *camera, GPContext *context, unsigned char cmd,
 				       (ret_data[1] == 0x00)) {
 			gp_context_error (context, _("Camera is in wrong "
 				"mode. Please contact "
-				"<gphoto-devel@gphoto.org>."));
+				"%s."), MAIL_GPHOTO_DEVEL);
 			return (GP_ERROR);
 		}
 
@@ -348,12 +347,12 @@ ricoh_transmit (Camera *camera, GPContext *context, unsigned char cmd,
 				       (ret_data[1] == 0x00)) {
 			gp_context_error (context, _("Camera did not "
 				"accept the parameters. Please contact "
-				"<gphoto-devel@gphoto.org>."));
+				"%s."), MAIL_GPHOTO_DEVEL);
 			return (GP_ERROR);
 		}
 
 		gp_context_error (context, _("An unknown error occurred. "
-			"Please contact <gphoto-devel@gphoto.org>."));
+			"Please contact %s."), MAIL_GPHOTO_DEVEL);
 		return (GP_ERROR);
         }
 
