@@ -1,6 +1,6 @@
 /* library.c
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
+ * Copyright  2001 Lutz Mller <lutz@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
 #include "library.h"
 
 #include <stdio.h>
@@ -136,11 +136,14 @@ int sierra_list_files (Camera *camera, const char *folder, CameraList *list, GPC
 	/*
 	 * This is a non-fatal check if a memory card is present. Some
 	 * cameras don't understand this command and error out here.
+	 * Since some cameras do not like it at all, do not send it for those.
 	 */
-	r = sierra_get_int_register (camera, 51, &i, NULL);
-	if ((r >= 0) && (i == 1)) {
-		gp_context_error (context, _("No memory card present"));
-		return GP_ERROR_NOT_SUPPORTED;
+	if (!(camera->pl->flags & SIERRA_NO_51)) {
+		r = sierra_get_int_register (camera, 51, &i, NULL);
+		if ((r >= 0) && (i == 1)) {
+			gp_context_error (context, _("No memory card present"));
+			return GP_ERROR_NOT_SUPPORTED;
+		}
 	}
 
 	/* We need to change to the folder first */
@@ -233,7 +236,7 @@ int sierra_list_folders (Camera *camera, const char *folder, CameraList *list,
 int sierra_get_picture_folder (Camera *camera, char **folder)
 {
 	int i;
-	CameraList list;
+	CameraList *list;
 	const char *name = NULL;
 
 	GP_DEBUG ("* sierra_get_picture_folder");
@@ -248,11 +251,13 @@ int sierra_get_picture_folder (Camera *camera, char **folder)
 		return GP_OK;
 	}
 
+	CHECK (gp_list_new (&list));
+
 	/* It is assumed that the camera conforms with the JEIDA standard .
 	   Thus, look for the picture folder into the /DCIM directory */
-	CHECK (gp_filesystem_list_folders (camera->fs, "/DCIM", &list, NULL) );
-	for(i=0 ; i < gp_list_count (&list) ; i++) {
-		CHECK (gp_list_get_name (&list, i, &name) );
+	CHECK (gp_filesystem_list_folders (camera->fs, "/DCIM", list, NULL) );
+	for(i=0 ; i < gp_list_count (list) ; i++) {
+		CHECK (gp_list_get_name (list, i, &name) );
 		GP_DEBUG ("* check folder %s", name);
 		if (isdigit(name[0]) && isdigit(name[1]) && isdigit(name[2]))
 			break;
@@ -263,10 +268,12 @@ int sierra_get_picture_folder (Camera *camera, char **folder)
 		*folder = (char*) calloc(strlen(name)+7, sizeof(char));
 		strcpy(*folder, "/DCIM/");
 		strcat(*folder, name);
+		gp_list_free (list);
 		return GP_OK;
-	}
-	else
+	} else {
+		gp_list_free (list);
 		return GP_ERROR_DIRECTORY_NOT_FOUND;
+	}
 }
 
 /**
@@ -437,7 +444,7 @@ sierra_write_packet (Camera *camera, char *packet, GPContext *context)
 		packet[length-1] = (checksum >> 8) & 0xff; 
 	}
 
-	if (camera->pl->usb_wrap) {
+	if (camera->pl->flags & SIERRA_WRAP_USB) {
 		CHECK (usb_wrap_write_packet (camera->port, packet, length));
 	} else {
 		CHECK (gp_port_write (camera->port, packet, length));
@@ -456,7 +463,7 @@ sierra_write_nak (Camera *camera, GPContext *context)
 
         buf[0] = SIERRA_PACKET_NAK;
         ret = sierra_write_packet (camera, buf, context);
-        if (camera->port->type == GP_PORT_USB && !camera->pl->usb_wrap)
+        if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
                 gp_port_usb_clear_halt(camera->port, GP_PORT_USB_ENDPOINT_IN);
         return (ret);
 }
@@ -515,7 +522,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 
 		/* Clear the USB bus
 		   (what about the SERIAL bus : do we need to flush it?) */
-		if (camera->port->type == GP_PORT_USB && !camera->pl->usb_wrap)
+		if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
 			gp_port_usb_clear_halt (camera->port,
 						GP_PORT_USB_ENDPOINT_IN);
 
@@ -523,7 +530,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		 * Read data through the bus. If an error occurred,
 		 * try again.
 		 */
-		if (camera->port->type == GP_PORT_USB && camera->pl->usb_wrap)
+		if (camera->port->type == GP_PORT_USB && (camera->pl->flags & SIERRA_WRAP_USB))
 			result = usb_wrap_read_packet (camera->port, packet,
 						       blocksize);
 		else
@@ -533,7 +540,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 				  gp_result_as_string (result));
 			if (++r > 2) {
 				if (camera->port->type == GP_PORT_USB &&
-				    !camera->pl->usb_wrap)
+				    !(camera->pl->flags & SIERRA_WRAP_USB))
 					gp_port_usb_clear_halt (camera->port,
 						GP_PORT_USB_ENDPOINT_IN);
 				GP_DEBUG ("Giving up...");
@@ -560,7 +567,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 
 			/* Those are all single byte packets. */
 			if (camera->port->type == GP_PORT_USB &&
-			    !camera->pl->usb_wrap)
+			    !(camera->pl->flags & SIERRA_WRAP_USB))
 				gp_port_usb_clear_halt (camera->port,
 						GP_PORT_USB_ENDPOINT_IN);
 
@@ -582,9 +589,10 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 			 */
 			gp_context_error (context, _("The first byte "
 				"received (0x%x) is not valid."), packet[0]);
-			while (gp_port_read (camera->port, packet, 1) >= 0);
+			while (gp_port_read (camera->port, packet, 1) > 0)
+				;
 			if (camera->port->type == GP_PORT_USB &&
-			    !camera->pl->usb_wrap)
+			    !(camera->pl->flags & SIERRA_WRAP_USB))
 				gp_port_usb_clear_halt (camera->port,
 						GP_PORT_USB_ENDPOINT_IN);
 			return (GP_ERROR_CORRUPTED_DATA);
@@ -600,7 +608,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 					       packet + br, 4 - br);
 			if (result < 0) {
 				if (camera->port->type == GP_PORT_USB &&
-				    !camera->pl->usb_wrap)
+				    !(camera->pl->flags & SIERRA_WRAP_USB))
 					gp_port_usb_clear_halt (camera->port,
 						GP_PORT_USB_ENDPOINT_IN);
 				GP_DEBUG ("Could not read length of "
@@ -673,7 +681,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		r++;
 		if (r + 1 >= RETRIES) {
 			if (camera->port->type == GP_PORT_USB &&
-			    !camera->pl->usb_wrap)
+			    !(camera->pl->flags & SIERRA_WRAP_USB))
 				gp_port_usb_clear_halt (camera->port,
 						GP_PORT_USB_ENDPOINT_IN);
 			GP_DEBUG ("Giving up...");
@@ -685,7 +693,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		GP_SYSTEM_SLEEP (10);
 	}
 
-	if (camera->port->type == GP_PORT_USB && !camera->pl->usb_wrap)
+	if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
 		gp_port_usb_clear_halt(camera->port, GP_PORT_USB_ENDPOINT_IN);
 
 	return GP_OK;
@@ -706,7 +714,7 @@ sierra_read_packet_wait (Camera *camera, char *buf, GPContext *context)
 				gp_context_error (context, _("Transmission "
 					"of packet timed out even after "
 					"%i retries. Please contact "
-					"<gphoto-devel@gphoto.org>."), r);
+					"%s."), r, MAIL_GPHOTO_DEVEL);
 				return GP_ERROR;
 			}
 			GP_DEBUG ("Retrying...");
@@ -754,7 +762,7 @@ sierra_transmit_ack (Camera *camera, char *packet, GPContext *context)
 		case SIERRA_PACKET_INVALID:
 			gp_context_error (context, _("Packet got rejected "
 				"by camera. Please contact "
-				"<gphoto-devel@gphoto.org>."));
+				"%s."), MAIL_GPHOTO_DEVEL);
 			return GP_ERROR;
 
 		case SIERRA_PACKET_SESSION_END:
@@ -782,7 +790,7 @@ sierra_transmit_ack (Camera *camera, char *packet, GPContext *context)
 				gp_context_error (context, _("Could not "
 					"transmit packet (error code %i). "
 					"Please contact "
-					"<gphoto-devel@gphoto.org>."), buf[0]);
+					"%s."), buf[0], MAIL_GPHOTO_DEVEL);
 				return GP_ERROR;
 			}
 			GP_DEBUG ("Did not receive ACK. Retrying...");
@@ -827,7 +835,7 @@ sierra_write_ack (Camera *camera, GPContext *context)
 	
 	buf[0] = ACK;
 	ret = sierra_write_packet (camera, buf, context);
-	if (camera->port->type == GP_PORT_USB && !camera->pl->usb_wrap)
+	if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
 		gp_port_usb_clear_halt (camera->port, GP_PORT_USB_ENDPOINT_IN);
 	CHECK (ret);
 
@@ -892,8 +900,8 @@ sierra_init (Camera *camera, GPContext *context)
 				gp_context_error (context,
 					_("Got unexpected result "
 					  "0x%x. Please contact "
-					  "<gphoto-devel@gphoto.org>."),
-					buf[0]);
+					  "%s."),
+					buf[0], MAIL_GPHOTO_DEVEL);
 				return GP_ERROR;
 			}
 			break;
@@ -989,7 +997,7 @@ int sierra_sub_action (Camera *camera, SierraAction action, int sub_action,
 		return (GP_OK);
 	default:
 		gp_context_error (context, _("Received unexpected answer "
-		"(%i). Please contact <gphoto-devel@gphoto.org>."), buf[0]);
+		"(%i). Please contact %s."), buf[0], MAIL_GPHOTO_DEVEL);
 		return (GP_ERROR);
 	}
 
@@ -1052,7 +1060,7 @@ int sierra_get_int_register (Camera *camera, int reg, int *value, GPContext *con
 		case SIERRA_PACKET_INVALID:
 			gp_context_error (context, _("Could not get "
 				"register %i. Please contact "
-				"<gphoto-devel@gphoto.org>."), reg);
+				"%s."), reg, MAIL_GPHOTO_DEVEL);
 			return GP_ERROR;
 		case SIERRA_PACKET_DATA_END:
 			r = ((unsigned char) buf[4]) +
@@ -1112,7 +1120,8 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 	char packet[4096];
 	char type;
 	long int x=0;
-	int seq=0, size=0, do_percent;
+	int seq=0, size=0; 
+	int do_percent;
 	unsigned int id = 0;
 
 	GP_DEBUG ("sierra_set_string_register: reg %i, value '%s'", reg, s);
@@ -1120,7 +1129,6 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 	/* Make use of the progress bar when the packet is "large enough" */
 	if (length > MAX_DATA_FIELD_LENGTH) {
 		do_percent = 1;
-#ifdef RECURSION_REMOVED
 		/*
 		 * This code can lead to recursive calls in gtkam - the
 		 * main problem is when thumbnails are being downloaded,
@@ -1131,7 +1139,6 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 		 */
 		id = gp_context_progress_start (context, length,
 						_("Sending data..."));
-#endif
 	}
 	else
 		do_percent = 0;
@@ -1164,10 +1171,8 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 
 		/* Transmit packet */
 		CHECK (sierra_transmit_ack (camera, packet, context));
-#ifdef RECURSION_REMOVED
 		if (do_percent)
 			gp_context_progress_update (context, id, x);
-#endif
 	}
 	if (do_percent)
 		gp_context_progress_stop (context, id);
@@ -1180,20 +1185,21 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 				unsigned int *b_len, GPContext *context)
 {
 	unsigned char p[34816];
-	const char *file_name;
-	unsigned int packlength, total = *b_len;
-	unsigned int id = 0;
+	unsigned int packlength, total = (b_len) ? *b_len : 0;
 	int retries, r;
+	unsigned int min_progress_bytes;
 	static int in_function = 0;
+	const char *file_name;
+	unsigned int id = 0;
 
-	GP_DEBUG ("sierra_get_string_register:  reg %i, file number %i,"
-		  " ext protocol %x", reg, fnumber,
-		  camera->pl->use_extended_protocol);
+	GP_DEBUG ("sierra_get_string_register:  reg %i, file number %i, "
+		  " total %d, flags 0x%x", reg, fnumber, total,
+		  camera->pl->flags);
 
 	if (in_function != 0) {
 		gp_context_error (context, _("recursive calls are not"
 			" supported by the sierra driver! Please contact"
-			" <gphoto-devel@gphoto.org>."));
+			" %s."), MAIL_GPHOTO_DEVEL);
 		return GP_ERROR;
 	}
 	in_function = 1;
@@ -1209,11 +1215,17 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 	 * and receive 32k size packages, otherwise code 0x04 is used and
 	 * we have 2k size packages.
 	 */
-	p[4] = camera->pl->use_extended_protocol ? 0x06 : 0x04;
+	if (camera->pl->flags & SIERRA_EXT_PROTO) {
+		p[4] = 0x06;
+		min_progress_bytes = 32 * 1024;
+	} else {
+		p[4] = 0x04;
+		min_progress_bytes = 2 * 1024;
+	}
 	p[5] = reg;
 	CHECK (sierra_write_packet (camera, p, context));
 
-	if (file) {
+	if (file && total > min_progress_bytes) {
 		CHECK (gp_file_get_name(file, &file_name));
 		id = gp_context_progress_start (context, total, file_name);
 	}
@@ -1242,7 +1254,7 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 		case SIERRA_PACKET_INVALID:
 			gp_context_error (context, _("Could not get "
 				"string register %i. Please contact "
-				"<gphoto-devel@gphoto.org>."), reg);
+				"%s."), reg, MAIL_GPHOTO_DEVEL);
 			in_function = 0;
 			return GP_ERROR;
 		default:
@@ -1258,17 +1270,17 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 		*b_len += packlength;
 
 		if (file) {
-			CHECK (gp_file_append (file, &p[4], packlength));
-#ifdef GTKAM_IS_MODIFIED
+		    CHECK (gp_file_append (file, &p[4], packlength));
+		    if (total > min_progress_bytes)
 			gp_context_progress_update (context, id, *b_len);
-#endif
 		}
 
 	} while (p[0] != SIERRA_PACKET_DATA_END);
-	if (file)
+	if (file && total > min_progress_bytes)
 		gp_context_progress_stop (context, id);
 
-	GP_DEBUG ("sierra_get_string_register: completed OK");
+	GP_DEBUG ("sierra_get_string_register: completed OK, *b_len %d",
+		  *b_len);
 	in_function = 0;
 	return GP_OK;
 }
@@ -1335,11 +1347,14 @@ sierra_capture (Camera *camera, CameraCaptureType type,
 	/*
 	 * This is a non-fatal check if a memory card is present. Some
 	 * cameras don't understand this command and error out here.
+	 * Since some cameras do not like it at all, do not send it for those.
 	 */
-	r = sierra_get_int_register (camera, 51, &n, context);
-	if ((r >= 0) && (n == 1)) {
-		gp_context_error (context, _("No memory card present"));
-		return GP_ERROR_NOT_SUPPORTED;
+	if (!(camera->pl->flags & SIERRA_NO_51)) {
+		r = sierra_get_int_register (camera, 51, &n, context);
+		if ((r >= 0) && (n == 1)) {
+			gp_context_error (context, _("No memory card present"));
+			return GP_ERROR_NOT_SUPPORTED;
+		}
 	}
 
 	/*
@@ -1419,12 +1434,57 @@ int sierra_get_pic_info (Camera *camera, unsigned int n,
 {
 	unsigned char buf[1024];
 	unsigned int buf_len = 0;
+	int value, audio_info[8];
 
 	CHECK (sierra_get_string_register (camera, 47, n, NULL, buf,
 					   &buf_len, context));
+	if (buf_len == 0) {
+		/*
+		 * Many (older) serial cameras seem to return 0 bytes back
+		 * and effectively do not support register 47. Try using
+		 * other registers to get the data.
+		 *
+		 * Don't fail on error so unsupported or semi-broken
+		 * behaviour does not prevent downloads or other use.
+		 */
+		memset(pic_info, 0, sizeof(*pic_info));
+		if (sierra_get_size(camera, 12, n, &value, context) == GP_OK) {
+			pic_info->size_file = value;
+		}
+		if (sierra_get_size(camera, 13, n, &value, context) == GP_OK) {
+			pic_info->size_preview = value;
+		}
+		/*
+		 * If the following hangs any cameras, just delete this
+		 * code, and assume that cameras with audio will get audio
+		 * file information via reg 47.
+		 *
+		 * Note that the value variable as used here is number of
+		 * bytes transferred, not the returned data. Some
+		 * unsupported sierra commands return zero bytes.
+		 */
+		if ((sierra_get_string_register (camera, 43, n, NULL,
+				(unsigned char *) &audio_info, &value,
+				context) == GP_OK) && (value != 0)) {
+			pic_info->size_audio = audio_info[0];
+		}
+		if (sierra_get_int_register (camera, 39, &value, context)
+		    == GP_OK) {
+			pic_info->locked = value;
+		} else {
+			/*
+			 * Default to marking as locked (not-deletable).
+			 * It looks the file can still be deleted if the
+			 * file is not really locked.
+			 */
+			pic_info->locked = SIERRA_LOCKED_YES;
+		}
+		return (GP_OK);
+	}
+
 	if (buf_len != 32) {
 		gp_context_error (context, _("Expected 32 bytes, got %i. "
-			"Please contact <gphoto-devel@gphoto.org>."), buf_len);
+			"Please contact %s."), buf_len, MAIL_GPHOTO_DEVEL);
 		return (GP_ERROR_CORRUPTED_DATA);
 	}
 
@@ -1453,9 +1513,15 @@ int sierra_set_locked (Camera *camera, unsigned int n, SierraLocked locked,
 		       GPContext *context)
 {
 	CHECK (sierra_set_int_register (camera, 4, n, context));
+	CHECK (sierra_sub_action (camera, SIERRA_ACTION_PROT_STATE,
+				  (int) locked, context));
+	return GP_OK;
+}
 
-	gp_context_error (context, _("Not implemented!"));
-	return GP_ERROR;
-
+int sierra_get_size (Camera *camera, int reg, unsigned int n, int *value,
+		     GPContext *context)
+{
+	CHECK (sierra_set_int_register (camera, 4, n, context));
+	CHECK (sierra_get_int_register (camera, reg, value, context));
 	return GP_OK;
 }
