@@ -1,6 +1,6 @@
 /* sierra.c:
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
+ * Copyright  2001 Lutz Mller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,10 +17,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
 #include "sierra.h"
 
 #include <stdlib.h>
+#include <_stdint.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
@@ -60,104 +61,125 @@ const char JPEG_SOF_MARKER[]  = { (char)0xFF, (char)0xD9, '\0' };
 const char JPEG_APP1_MARKER[] = { (char)0xFF, (char)0xE1, '\0' };
 const char TIFF_SOI_MARKER[]  = { (char)0x49, (char)0x49, (char)0x2A, (char)0x00, (char)0x08, '\0' };
 
-static SierraCamera sierra_cameras[] = {
+static struct {
+	const char *manuf;
+	const char *model;
+	SierraModel sierra_model;
+	int  usb_vendor;
+	int  usb_product;
+	SierraFlags  flags;
+	struct CameraDesc const *cam_desc;
+} sierra_cameras[] = {
 	/* Camera Model, Sierra Model, USB(vendor id, product id,
 	   USB wrapper protocol) */ 
-	{"Agfa ePhoto 307", 	SIERRA_MODEL_DEFAULT, 	0, 0, 0 },
-	{"Agfa ePhoto 780", 	SIERRA_MODEL_DEFAULT, 	0, 0, 0 },
-	{"Agfa ePhoto 780C", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Agfa ePhoto 1280", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Agfa ePhoto 1680", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Apple QuickTake 200", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Chinon ES-1000", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Epson PhotoPC 500", 	SIERRA_MODEL_EPSON,	0, 0, 0 },
-	{"Epson PhotoPC 550", 	SIERRA_MODEL_EPSON,	0, 0, 0 },
-	{"Epson PhotoPC 600", 	SIERRA_MODEL_EPSON,	0, 0, 0 },
-	{"Epson PhotoPC 700", 	SIERRA_MODEL_EPSON,	0, 0, 0 },
-	{"Epson PhotoPC 800", 	SIERRA_MODEL_EPSON,	0, 0, 0 },
-	{"Epson PhotoPC 850z",  SIERRA_MODEL_EPSON,     0x4b8, 0x402, 0},
-	{"Epson PhotoPC 3000z", SIERRA_MODEL_CAM_DESC,	0x4b8, 0x403, 0, &ep3000z_cam_desc},
-	{"Nikon CoolPix 100", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 300", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 700", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 800", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 880",	SIERRA_MODEL_CAM_DESC,	0x04b0, 0x0103, 0,
+	{"Agfa", "ePhoto 307", 	SIERRA_MODEL_DEFAULT, 	0, 0,
+					SIERRA_NO_51 },
+	{"Agfa", "ePhoto 780", 	SIERRA_MODEL_DEFAULT, 	0, 0, 0 },
+	{"Agfa", "ePhoto 780C",	SIERRA_MODEL_DEFAULT,	0, 0,
+					SIERRA_NO_51 | SIERRA_LOW_SPEED },
+	{"Agfa", "ePhoto 1280",	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Agfa", "ePhoto 1680", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Apple", "QuickTake 200", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Chinon", "ES-1000", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Epson", "PhotoPC 500", SIERRA_MODEL_EPSON,	0, 0, 0 },
+	{"Epson", "PhotoPC 550", SIERRA_MODEL_EPSON,	0, 0, 0 },
+	{"Epson", "PhotoPC 600", SIERRA_MODEL_EPSON,	0, 0, 0 },
+	{"Epson", "PhotoPC 650", SIERRA_MODEL_EPSON,	0, 0,
+					SIERRA_NO_51 },
+	{"Epson", "PhotoPC 700", SIERRA_MODEL_EPSON,	0, 0, 0 },
+	{"Epson", "PhotoPC 800", SIERRA_MODEL_EPSON,	0, 0, 0 },
+	{"Epson", "PhotoPC 850z", SIERRA_MODEL_EPSON,     0x4b8, 0x402, 0},
+	{"Epson", "PhotoPC 3000z", SIERRA_MODEL_CAM_DESC,	0x4b8, 0x403, 0, &ep3000z_cam_desc},
+	{"Nikon", "CoolPix 100", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 300", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 700", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 800", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 880",SIERRA_MODEL_CAM_DESC,	0x04b0, 0x0103, 0,
 							&cp880_cam_desc},
-	{"Nikon CoolPix 900", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 900S", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 910", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 950", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 950S", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Nikon CoolPix 990",	SIERRA_MODEL_DEFAULT,	0x04b0, 0x0102, 0},
-	{"Nikon CoolPix 995",   SIERRA_MODEL_CAM_DESC,  0x04b0, 0x0104, 1,
+	{"Nikon", "CoolPix 900", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 900S", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 910", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 950", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 950S", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Nikon", "CoolPix 990", SIERRA_MODEL_DEFAULT,	0x04b0, 0x0102, 0},
+	{"Nikon", "CoolPix 995",   SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0104,
+						SIERRA_WRAP_USB,
 							&cp995_cam_desc},
-	{"Olympus D-100Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-200L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-220L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-300L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-320L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-330R", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-340L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-340R", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-360L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-400L Zoom", SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-450Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-460Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-500L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-600L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-600XL", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus D-620L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-400", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-400L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-410", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-410L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-420", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-420L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-700UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105, 1,
+	{"Olympus", "D-100Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-200L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-220L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-300L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-320L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-330R", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-340L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-340R", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-360L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-400L Zoom", SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-450Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-460Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-500L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-600L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-600XL", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "D-620L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-400", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-400L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-410", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-410L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-420", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-420L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-700UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
+					SIERRA_WRAP_USB, &oly3040_cam_desc},
+	{"Olympus", "C-750UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
+					SIERRA_WRAP_USB, &oly750uz_cam_desc},
+	{"Olympus", "C-800", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-800L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-820", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-820L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-830L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-840L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-860L",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-900 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-900L Zoom", SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-1000L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-1400L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-1400XL", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-2000Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-2020Z",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-2040Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
+					SIERRA_WRAP_USB, &oly3040_cam_desc},
+	{"Olympus", "C-2100UZ", SIERRA_MODEL_CAM_DESC, 0x07b4, 0x100, 0,
+					&oly3040_cam_desc},
+	{"Olympus", "C-2500L",     SIERRA_MODEL_OLYMPUS,   0, 0, 0 },
+	/*
+	 * Note: The 3000z can't be auto selected, as it has the same USB
+	 * id as the 2040 (and others, but the others are still using the
+	 * the 3040 desc), so you have to set the camera "model" via the
+	 * front end (gtkam or gphoto2 command line) in order to use it.
+	 */
+	{"Olympus", "C-3000Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
+							&oly3000z_cam_desc},
+	{"Olympus", "C-3020Z", SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
+					SIERRA_WRAP_USB, &oly3040_cam_desc},
+	{"Olympus", "C-3030Z", SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
 							&oly3040_cam_desc},
-	{"Olympus C-800", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-800L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-820", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-820L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-830L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-840L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-860L",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-900 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-900L Zoom", SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-1000L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-1400L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-1400XL", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-2000Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-2020Z",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus C-2040Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105, 1,
-							&oly3040_cam_desc},
-	{"Olympus C-2100UZ",    SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
-							&oly3040_cam_desc},
-	{"Olympus C-2500L",     SIERRA_MODEL_OLYMPUS,   0, 0, 0 },
-/* Does this model really exist?
- *	{"Olympus C-2500Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
- */
-	{"Olympus C-3000Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
-							&oly3040_cam_desc},
-	{"Olympus C-3020Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105, 1,
-							&oly3040_cam_desc},
-	{"Olympus C-3030Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
-							&oly3040_cam_desc},
-	{"Olympus C-3040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105, 1,
-							&oly3040_cam_desc},
-	{"Olympus C-4040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105, 1,
-							&oly3040_cam_desc},
-	{"Panasonic Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Polaroid PDC 640", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Polaroid PDC 2300Z",	SIERRA_MODEL_DEFAULT,   0, 0, 0 },
-	{"Sanyo DSC-X300", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sanyo DSC-X350", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sanyo VPC-G200", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sanyo VPC-G200EX", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sanyo VPC-G210", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sanyo VPC-G250", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sierra Imaging SD640",SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"", 			SIERRA_MODEL_DEFAULT,	0, 0, 0 }
+	{"Olympus", "C-3040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
+					SIERRA_WRAP_USB, &oly3040_cam_desc},
+	{"Olympus", "C-4040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
+					SIERRA_WRAP_USB, &oly3040_cam_desc},
+	{"Olympus", "C-5050Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
+					SIERRA_WRAP_USB, &oly3040_cam_desc},
+	{"Panasonic", "Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT, 0, 0, 0 },
+	{"Polaroid", "PDC 640", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Polaroid", "PDC 2300Z", SIERRA_MODEL_DEFAULT, 0x0546, 0x0daf,
+					SIERRA_SKIP_INIT },
+	{"Sanyo", "DSC-X300", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sanyo", "DSC-X350", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sanyo", "VPC-G200", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sanyo", "VPC-G200EX", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sanyo", "VPC-G210", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sanyo", "VPC-G250", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sierra Imaging", "SD640",SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{NULL, NULL, 0,	0, 0, 0}
 };
 
 static struct {
@@ -184,9 +206,11 @@ int camera_abilities (CameraAbilitiesList *list)
 	int x;
 	CameraAbilities a;
 
-	for (x = 0; strlen (sierra_cameras[x].model) > 0; x++) {
+	for (x = 0; sierra_cameras[x].manuf; x++) {
 		memset(&a, 0, sizeof(a));
-		strcpy (a.model, sierra_cameras[x].model);
+		strcpy (a.model, sierra_cameras[x].manuf);
+		strcat (a.model, ":");
+		strcat (a.model, sierra_cameras[x].model);
 		a.status = GP_DRIVER_STATUS_PRODUCTION;
 		a.port     = GP_PORT_SERIAL;
 		if ((sierra_cameras[x].usb_vendor  > 0) &&
@@ -195,9 +219,13 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.speed[0] = 9600;
 		a.speed[1] = 19200;
 		a.speed[2] = 38400;
-		a.speed[3] = 57600;
-		a.speed[4] = 115200;
-		a.speed[5] = 0;
+		if (!(sierra_cameras[x].flags & SIERRA_LOW_SPEED)) {
+			a.speed[3] = 57600;
+			a.speed[4] = 115200;
+			a.speed[5] = 0;
+		} else {
+			a.speed[3] = 0;
+		}
 		a.operations        = 	GP_OPERATION_CAPTURE_IMAGE |
 					GP_OPERATION_CAPTURE_PREVIEW |
 					GP_OPERATION_CONFIG;
@@ -242,7 +270,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 {
 	Camera *camera = data;
 	int n;
-	SierraPicInfo pic_info;
+	SierraPicInfo i;
 
 	/*
 	 * Get the file number from the CameraFilesystem. We need numbers
@@ -255,30 +283,37 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	info->file.fields    = GP_FILE_INFO_NONE;
 	info->preview.fields = GP_FILE_INFO_NONE;
 	info->audio.fields   = GP_FILE_INFO_NONE;
+	info->file.permissions = GP_FILE_PERM_READ;
 
 	/* Name of image */
 	strncpy (info->file.name, filename, sizeof (info->file.name) - 1);
 	info->file.name[sizeof (info->file.name) - 1] = '\0';
 	info->file.fields |= GP_FILE_INFO_NAME;
 
-	/* Get information about this image */
+	/*
+	 * Get information about this image. Don't make this fatal as
+	 * at least the "Agfa ePhoto307" doesn't support this command.
+	 */
 	CHECK (camera_start (camera, context));
 	CHECK_STOP (camera, sierra_change_folder (camera, folder, context));
-	CHECK_STOP (camera, sierra_get_pic_info (camera, n, &pic_info, context));
-
+	memset (&i, 0, sizeof (SierraPicInfo));
+	CHECK_STOP (camera, sierra_get_pic_info (camera, n, &i, context));
 	/* Size of file */
-	info->file.fields |= GP_FILE_INFO_SIZE;
-	info->file.size = pic_info.size_file;
+	if (i.size_file) {
+		info->file.fields |= GP_FILE_INFO_SIZE;
+		info->file.size = i.size_file;
+	}
 
 	/* Size of preview */
-	info->preview.fields |= GP_FILE_INFO_SIZE;
-	info->preview.size = pic_info.size_preview;
+	if (i.size_preview) {
+		info->preview.fields |= GP_FILE_INFO_SIZE;
+		info->preview.size = i.size_preview;
+	}
 
 	/* Audio data? */
-	if (pic_info.size_audio) {
-
+	if (i.size_audio) {
 		/* Size */
-		info->audio.size = pic_info.size_audio;
+		info->audio.size = i.size_audio;
 		info->audio.fields |= GP_FILE_INFO_SIZE;
 
 		/* Type */
@@ -302,8 +337,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 
 	/* Image protected? */
 	info->file.fields |= GP_FILE_INFO_PERMISSIONS;
-	info->file.permissions = GP_FILE_PERM_READ;
-	if (pic_info.locked == SIERRA_LOCKED_NO)
+	if (i.locked == SIERRA_LOCKED_NO)
 		info->file.permissions |= GP_FILE_PERM_DELETE;
 
 	return (camera_stop (camera, context));
@@ -315,23 +349,23 @@ set_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 {
 	Camera *camera = data;
 	int n;
-	SierraPicInfo pic_info;
+	SierraPicInfo i;
 
 	CHECK (n = gp_filesystem_number (camera->fs, folder, filename, context));
 	n++;
 
 	CHECK (camera_start (camera, context));
 	CHECK_STOP (camera, sierra_change_folder (camera, folder, context));
-	CHECK_STOP (camera, sierra_get_pic_info (camera, n, &pic_info, context));
+	CHECK_STOP (camera, sierra_get_pic_info (camera, n, &i, context));
 
 	if (info.file.fields & GP_FILE_INFO_PERMISSIONS) {
 		if (info.file.permissions & GP_FILE_PERM_DELETE) {
-			if (pic_info.locked == SIERRA_LOCKED_YES) {
+			if (i.locked == SIERRA_LOCKED_YES) {
 				CHECK_STOP (camera, sierra_set_locked (camera,
 						n, SIERRA_LOCKED_NO, context));
 			}
 		} else {
-			if (pic_info.locked == SIERRA_LOCKED_NO) {
+			if (i.locked == SIERRA_LOCKED_NO) {
 				CHECK_STOP (camera, sierra_set_locked (camera,
 						n, SIERRA_LOCKED_YES, context));
 			}
@@ -421,7 +455,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	int jpeg_size;
 	const char *data, *mime_type;
 	long int size;
-	SierraPicInfo info;
+	int download_size, audio_info[8], transferred;
 
 	/*
 	 * Get the file number from the CameraFileSystem.
@@ -450,12 +484,46 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	CHECK (camera_start (camera, context));
 	CHECK_STOP (camera, sierra_change_folder (camera, folder, context));
 
-	/* We need the file size in order to display progress information */
-	CHECK_STOP (camera, sierra_get_pic_info (camera, n, &info, context));
+	/*
+	 * We need the file size in order to display progress information.
+	 * Since we _can_ run without progress information, make failures
+	 * here not prevent the download - but the failure should still
+	 * show up in any logs.
+	 *
+	 * Some cameras do not return anything useful for register 47, so
+	 * don't try to use it at all (via sierra_get_pic_info) here.
+	 */
+	download_size = 0;
+	switch (type) {
+	case GP_FILE_TYPE_PREVIEW:
+	case GP_FILE_TYPE_EXIF:
+		sierra_get_size(camera, 13, n, &download_size, context);
+		break;
+	case GP_FILE_TYPE_NORMAL:
+		sierra_get_size(camera, 12, n, &download_size, context);
+		break;
+	case GP_FILE_TYPE_AUDIO:
+		sierra_get_string_register (camera, 43, n, NULL,
+			(unsigned char *) &audio_info, &transferred, context);
+		if (transferred == 0) 
+			download_size = 0;
+		else
+			download_size = audio_info[0];
+		break;
+	default:
+		return (GP_ERROR_NOT_SUPPORTED);
+	}
 
 	/* Get the file */
 	CHECK_STOP (camera, sierra_get_string_register (camera, regd, n,
-					file, NULL, &(info.size_file), context));
+					file, NULL, &download_size,  context));
+	if (download_size == 0) {
+		/*
+		 * Assume a failure. At least coolpix 880 returns no error
+		 * and no data when asked to download audio.
+		 */
+		return (GP_ERROR_NOT_SUPPORTED);
+	}
 	CHECK (camera_stop (camera, context));
 
 	/* Now get the data and do some post-processing */
@@ -555,9 +623,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	 * XXX the CHECK() macro's don't allow for cleanup on error.
 	 */
 	/* Get the file number from the CameraFilesystem */
-#ifdef GTKAM_IS_MODIFIED
 	gp_context_progress_update (context, id, 0);
-#endif
 	CHECK (n = gp_filesystem_number (camera->fs, folder, filename, context));
 	/* Set the working folder and delete the file */
 	CHECK (camera_start (camera, context));
@@ -663,12 +729,11 @@ put_file_func (CameraFilesystem * fs, const char *folder, CameraFile * file, voi
 }
 
 #ifdef UNUSED_CODE
-// FIXME: Is this function still usefull ?
 static void dump_register (Camera *camera, GPContext *context)
 {
 	int ret, value, i;
 	const char *description[] = {
-		"?",				//   0
+		"?",				/* 0 */
 		"resolution",
 		"date",
 		"shutter speed",
@@ -678,7 +743,7 @@ static void dump_register (Camera *camera, GPContext *context)
 		"flash mode",
 		"?",
 		"?",
-		"frames taken",			//  10
+		"frames taken",			/* 10 */
 		"frames left",
 		"size of current image",
 		"size of thumbnail of current image",
@@ -688,7 +753,7 @@ static void dump_register (Camera *camera, GPContext *context)
 		"?",
 		"?",
 		"brightness/contrast",
-		"white balance",		//  20
+		"white balance",		/* 20 */
 		"?",
 		"camera id",
 		"auto off (host)",
@@ -698,7 +763,7 @@ static void dump_register (Camera *camera, GPContext *context)
 		"?",
 		"memory left",
 		"?",
-		"?",				//  30
+		"?",				/* 30 */
 		"?",
 		"?",
 		"lens mode",
@@ -708,31 +773,31 @@ static void dump_register (Camera *camera, GPContext *context)
 		"?",
 		"lcd auto off",
 		"?",
-		"?",				//  40
+		"?",				/* 40 */
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
-		"?",				//  50
+		"?",				/* 50 */
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
-		"?",				//  60
+		"?",				/* 60 */
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
-		"spot metering mode",		//  70
+		"spot metering mode",		/* 70 */
 		"?",
 		"zoom",
 		"?", "?", "?", "?", "?", "?", 
 		"current filename",
-		"?",				//  80
+		"?",				/* 80 */
 		"?", "?",
 		"number of folders in current folder/folder number",
 		"current folder name",
 		"?", "?", "?", "?", "?",
-		"?",				//  90
+		"?",				/* 90 */
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
-		"?", 				// 100
+		"?", 				/* 100 */
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
-		"?", 				// 110
+		"?", 				/* 110 */
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
-		"?",				// 120
+		"?",				/* 120 */
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
-		"?"				// 130
+		"?"				/* 130 */
 	};
 		
 
@@ -836,9 +901,10 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
 	/* Color Mode */
         ret = sierra_get_int_register (camera, 6, &value, context);
         if (ret >= 0) {
-
-		// Those values are for a C-2020 Z. If your model differs, we
-		// have to distinguish models here...
+		/*
+		 * Those values are for a C-2020 Z. If your model differs,
+		 * we have to distinguish models here.
+		 */
 		gp_widget_new (GP_WIDGET_RADIO, _("Color Mode"), &child);
 		gp_widget_add_choice (child, _("Normal"));
 		gp_widget_add_choice (child, _("Black/White"));
@@ -1752,7 +1818,6 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 static int
 camera_get_config_default (Camera *camera, CameraWidget **window, GPContext *context)
 {
-	// This really should change in the near future...
 	return camera_get_config_olympus (camera, window, context);
 }
 
@@ -1760,7 +1825,6 @@ camera_get_config_default (Camera *camera, CameraWidget **window, GPContext *con
 static int
 camera_set_config_default (Camera *camera, CameraWidget *window, GPContext *context)
 {
-	// This really should change in the near futur...
 	return camera_set_config_olympus (camera, window, context);
 }
 
@@ -1785,14 +1849,16 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *c)
 	 * At least on PhotoPC 3000z, if no card is present near all
 	 * retrieved info are either unreadable or invalid...
 	 */
-	r = sierra_get_int_register(camera, 51, &v, c);
-	if ((r >= 0) && (v == 1)) {
-		strcpy (buf, _("Note: no memory card present, some"
-			       " values may be invalid\n"));
-		strcpy (summary->text, buf);
+	if (!(camera->pl->flags & SIERRA_NO_51)) {
+		r = sierra_get_int_register(camera, 51, &v, c);
+		if ((r >= 0) && (v == 1)) {
+			strcpy (buf, _("Note: no memory card present, some"
+				       " values may be invalid\n"));
+			strcpy (summary->text, buf);
+		}
 	}
-
 	/* Get all the string-related info */
+
 	if (sierra_get_string_register (camera, 27, 0, NULL, t, &v, c) >= 0)
 		sprintf (buf, _("%sCamera Model: %s\n"), buf, t);
 	if (sierra_get_string_register (camera, 48, 0, NULL, t, &v, c) >= 0)
@@ -1833,13 +1899,13 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 	switch (camera->pl->model) {
 	case SIERRA_MODEL_CAM_DESC:
 		if (camera->pl->cam_desc->manual == NULL) {
-			strcpy (manual->text, "No camera manual available.\n");
+			strcpy (manual->text, _("No camera manual available.\n"));
 		} else {
-			strcpy (manual->text, camera->pl->cam_desc->manual);
+			strcpy (manual->text, _(camera->pl->cam_desc->manual));
 		}
 		break;
 	case SIERRA_MODEL_EPSON:
-		strcpy (manual->text,
+		snprintf (manual->text, sizeof(manual->text),
 			_("Some notes about Epson cameras:\n"
 			  "- Some parameters are not controllable remotely:\n"
 			  "  * zoom\n"
@@ -1847,7 +1913,7 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 			  "  * custom white balance setup\n"
 			  "- Configuration has been reverse-engineered with\n"
 			  "  a PhotoPC 3000z, if your camera acts differently\n"
-			  "  please send a mail to <gphoto-devel@gphoto.org> (in English)\n"));
+			  "  please send a mail to %s (in English)\n"), MAIL_GPHOTO_DEVEL);
 		break;
 	case SIERRA_MODEL_OLYMPUS:
 	default:
@@ -1939,10 +2005,9 @@ int
 camera_init (Camera *camera, GPContext *context) 
 {
         int x = 0, ret, value;
-        int vendor=0, product=0, usb_wrap=0;
+        int vendor=0;
 	GPPortSettings s;
 	CameraAbilities a;
-	const CameraDescType *cam_desc = NULL;
 
         /* First, set up all the function pointers */
         camera->functions->exit                 = camera_exit;
@@ -1958,17 +2023,20 @@ camera_init (Camera *camera, GPContext *context)
 
 	camera->pl->model = SIERRA_MODEL_DEFAULT;
 	camera->pl->first_packet = 1;
-	camera->pl->usb_wrap = 0;
-	camera->pl->use_extended_protocol = 0;
+	camera->pl->flags = 0;
 
 	/* Retrieve Camera information from name */
 	gp_camera_get_abilities (camera, &a);
-	for (x = 0; strlen (sierra_cameras[x].model) > 0; x++) {
-		if (!strcmp (sierra_cameras[x].model, a.model)) {
+	for (x = 0; sierra_cameras[x].manuf; x++) {
+		if (!strncmp (a.model, sierra_cameras[x].manuf,
+			      strlen (sierra_cameras[x].manuf)) &&
+		    !strcmp (a.model + strlen (sierra_cameras[x].manuf) + 1,
+			     sierra_cameras[x].model)) {
 			camera->pl->model = sierra_cameras[x].sierra_model;
 			vendor = sierra_cameras[x].usb_product;
-                        usb_wrap = sierra_cameras[x].usb_wrap;
-                        cam_desc = sierra_cameras[x].cam_desc;
+			camera->pl->flags = sierra_cameras[x].flags;
+			camera->pl->cam_desc = sierra_cameras[x].cam_desc;
+			break;
                }
 	}
 
@@ -1982,13 +2050,11 @@ camera_init (Camera *camera, GPContext *context)
 		camera->functions->set_config = camera_set_config_olympus;
 		break;
 	case SIERRA_MODEL_CAM_DESC:
-		if (cam_desc == NULL) {
+		if (camera->pl->cam_desc == NULL) {
 			GP_DEBUG ("*** sierra cam_desc NULL");
                         return (GP_ERROR_MODEL_NOT_FOUND);
 		}
-		camera->pl->cam_desc = cam_desc;
-		camera->pl->use_extended_protocol = 
-			cam_desc->use_extended_protocol;
+		camera->pl->flags |= camera->pl->cam_desc->flags;
 		camera->functions->get_config = camera_get_config_cam_desc;
 		camera->functions->set_config = camera_set_config_cam_desc;
 		break;
@@ -2013,14 +2079,19 @@ camera_init (Camera *camera, GPContext *context)
 		if (s.serial.speed)
 			camera->pl->speed = s.serial.speed;
 		else {
-			unsigned int speeds[] = {115200, 57600, 38400, 0}, i;
-			
-			for (i = 0; speeds[i]; i++) {
-				s.serial.speed = speeds[i];
+			int i;
+			/* Find the speed into abilities.speeds array.
+			 * Highest speeds are last ... so look backwards ...
+			 */
+			for (i=0;i<sizeof(a.speed)/sizeof(a.speed[0]) && a.speed[i];i++)
+				/*empty*/;
+			i--;
+			for (; i >= 0 ; i--) {
+				s.serial.speed = a.speed[i];
 				if (gp_port_set_settings (camera->port, s) >= 0)
 					break;
 			}
-			camera->pl->speed = (speeds[i] ? speeds[i] : 19200);
+			camera->pl->speed = (i>=0 ? a.speed[i] : 19200);
 		}
 
 		/* The camera defaults to 19200. */
@@ -2031,14 +2102,11 @@ camera_init (Camera *camera, GPContext *context)
         case GP_PORT_USB:
 
                 /* Test if we have usb information */
-                if ((vendor == 0) && (product == 0)) {
+                if (vendor == 0) {
                         free (camera->pl);
                         camera->pl = NULL;
                         return (GP_ERROR_MODEL_NOT_FOUND);
-                } else {
-			camera->pl->usb_wrap = usb_wrap;
-		}
-
+                }
 		/* Use the defaults the core parsed */
                 break;
 
@@ -2053,11 +2121,11 @@ camera_init (Camera *camera, GPContext *context)
         CHECK_FREE (camera, gp_port_set_timeout (camera->port, TIMEOUT));
 
 	/*
-	 * Send initialization sequence. Do not do this for the
-	 * 'Polaroid PDC 2300Z' (reported by William Bader
-	 * <williambader@hotmail.com>).
+	 * Send initialization sequence unless otherwise flagged. (The
+	 * 'Polaroid PDC 2300Z' fails sierra_init as reported by William
+	 * Bader <williambader@hotmail.com>).
 	 */
-	if (strcmp (a.model, "Polaroid PDC 2300Z"))
+	if (!(camera->pl->flags & SIERRA_SKIP_INIT))
 		CHECK (sierra_init (camera, context));
 
         /* Establish a connection */
@@ -2087,8 +2155,8 @@ camera_init (Camera *camera, GPContext *context)
         }
         CHECK_STOP_FREE (camera, gp_port_set_timeout (camera->port, TIMEOUT));
 
-        /* We are in "/" */
-        strcpy (camera->pl->folder, "/");
+        /* We start off not knowing where we are */
+        strcpy (camera->pl->folder, "");
         CHECK_STOP_FREE (camera, gp_filesystem_set_list_funcs (camera->fs,
 				file_list_func, folder_list_func, camera));
         CHECK_STOP_FREE (camera, gp_filesystem_set_info_funcs (camera->fs,
