@@ -1,4 +1,3 @@
-
 typedef enum {
 	SIERRA_MODEL_DEFAULT,   
 	SIERRA_MODEL_EPSON,
@@ -6,28 +5,26 @@ typedef enum {
 	SIERRA_MODEL_CAM_DESC,
 } SierraModel;
 
+typedef enum {
+	SIERRA_WRAP_USB	 = 1<<0,
+	SIERRA_NO_51	 = 1<<1,
+	SIERRA_LOW_SPEED = 1<<2,
+	SIERRA_EXT_PROTO = 1<<3,
+	SIERRA_SKIP_INIT = 1<<4,
+} SierraFlags;
+
 struct _CameraPrivateLibrary {
 	SierraModel model;
 	int folders;
 	int speed;
 	int first_packet;
-	int usb_wrap:1;  /* 0: packets sent "raw", 1: see sierra-usbwrap.h */
-	int use_extended_protocol:1;
+	SierraFlags flags;
 	struct CameraDesc const *cam_desc;
 	char folder[128];
 };
 
 struct CameraDescriptor;
 
-typedef struct {
-	char model[64];
-	SierraModel sierra_model;
-	int  usb_vendor;
-	int  usb_product;
-	int  usb_wrap;
-	struct CameraDesc const *cam_desc;
-} SierraCamera;
-
 #define CHECK(result)					\
 {							\
 	int res = (result);				\
