@@ -1,8 +1,8 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* gphoto2-port.c
  * 
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
- * Copyright (C) 1999 Scott Fritzinger <scottf@unr.edu>
+ * Copyright  2001 Lutz Mller <lutz@users.sf.net>
+ * Copyright  1999 Scott Fritzinger <scottf@unr.edu>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gphoto2-port.h"
 
 #include <stdarg.h>
@@ -471,9 +471,6 @@ gp_port_check_int_fast (GPPort *port, char *data, int size)
 {
         int retval;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Reading %i=0x%x bytes from inerrupt ep (fast)...",
-		size, size);
-
 	CHECK_NULL (port);
 	CHECK_INIT (port);
 
@@ -481,11 +478,28 @@ gp_port_check_int_fast (GPPort *port, char *data, int size)
 	CHECK_SUPP (port, _("check_int"), port->pc->ops->check_int);
 	retval = port->pc->ops->check_int (port, data, size, FAST_TIMEOUT);
 	CHECK_RESULT (retval);
-	if (retval != size)
+
+#ifdef IGNORE_EMPTY_INTR_READS
+	if (retval != size && retval != 0 )
+#else
+	if (retval != size )
+#endif
 		gp_log (GP_LOG_DEBUG, "gphoto2-port", "Could only read %i "
 			"out of %i byte(s)", retval, size);
 
-	gp_log_data ("gphoto2-port", data, retval);
+#ifdef IGNORE_EMPTY_INTR_READS
+	if ( retval != 0 ) {
+#endif
+		/* For Canon cameras, we will make lots of
+		   reads that will return zero length. Don't
+		   bother to log them as errors. */
+		gp_log (GP_LOG_DEBUG, "gphoto2-port",
+			"Reading %i=0x%x bytes from interrupt ep (fast)...",
+			size, size);
+		gp_log_data ("gphoto2-port", data, retval);
+#ifdef IGNORE_EMPTY_INTR_READS
+	}
+#endif
 
 	return (retval);
 }
